Attribute VB_Name = "PhCommon22"

Option Explicit

   ' The core Phidget class deals with functionality common to all
   ' Phidgets, such as opening and closing them, or setting Attach,
   ' Detach and Event handlers.
   '
   ' This class is also used to specify the associations between the
   ' Phidget software objects and their corresponding physical
   ' devices, and makes it possible to determine which Phidget is
   ' which in cases where it might otherwise be ambiguous.
   ' This class contains various functions such as Release, Retain,
   ' and getParent designed to be used with the Phidget Manager.
   ' These specialized functions may be safely ignored if your
   ' application does not require a Manager.  You can check the
   ' Manager API for more information.
   
   ' =======================================================================
   ' WORK IN PROGRESS !!!     === Contains ERRORS ===
   '
   ' Translation from Phidget22.h C++ code, to Visual Basic 6 code.
   '
   ' Most function comments contain the orginal Phidget22 C++ code.
   '
   ' =======================================================================
   ' Visual Basic V6.0 type-declaration characters
   ' $=STRING    %=INTEGER int16    &=LONG int32    !=SINGLE flo32
   ' #=DOUBLE flo64   @=Currency 64bit +/-922,337,203,685,477.5807 fixed "."
   ' =======================================================================
   '     VB Type      C++ Type
   '     --------------------------
   '     Byte         unsigned char
   '     Integer%     short
   '     Long&        int,uint32,long (unsigned int requires special handling)
   '     Single!      float
   '     Double#      double
   '     Currency@    int64
   ' =============================
   
   ' =======================================================================
   ' Enumerations:
   ' =======================================================================
    
   ' PhidgetReturnCode - The Result Codes Returned By All Functions
   '
   Global Const EPhidget_Ok = &H0               ' Success
   Global Const EPhidget_Perm = &H1             ' Not Permitted
   Global Const EPhidget_NoEnt = &H2            ' No Such Entity
   Global Const EPhidget_Timeout = &H3          ' Timed Out
   Global Const EPhidget_KeepAlive = &H3A       ' Keep Alive Failure
   Global Const EPhidget_Interrupted = &H4      ' Op Interrupted
   Global Const EPhidget_Io = &H5               ' Io Issue
   Global Const EPhidget_NoMemory = &H6         ' Memory Issue
   Global Const EPhidget_Access = &H7           ' Access (Permission) Issue
   Global Const EPhidget_Fault = &H8            ' Address Issue
   Global Const EPhidget_Busy = &H9             ' Resource Busy
   Global Const EPhidget_Exist = &HA            ' Object Exists
   Global Const EPhidget_NotDir = &HB           ' Object Is Not A Directory
   Global Const EPhidget_IsDir = &HC            ' Object Is A Directory
   Global Const EPhidget_InValid = &HD          ' InValid
   Global Const EPhidget_Nfile = &HE            ' Too Many Open Files In System
   Global Const EPhidget_Mfile = &HF            ' Too Many Open Files
   Global Const EPhidget_Nospc = &H10           ' Not Enough Space
   Global Const EPhidget_Fbig = &H11            ' File Too Big
   Global Const EPhidget_Rofs = &H12            ' Read Only Filesystem
   Global Const EPhidget_Ro = &H13              ' Read Only Object
   Global Const EPhidget_Unsupported = &H14     ' OpeRation Not Supported
   Global Const EPhidget_InValidArg = &H15      ' InValid Argument
   Global Const EPhidget_Again = &H16           ' Try Again
   Global Const EPhidget_NoTempty = &H1A        ' No Empty
   Global Const EPhidget_Unexpected = &H1C      ' Unexpected Error
   Global Const EPhidget_Duplicate = &H1B       ' Duplicate
   Global Const EPhidget_BadPassword = &H25     ' Bad Credential
   Global Const EPhidget_NetUnavail = &H2D      ' Network Unavailable
   Global Const EPhidget_ConnRef = &H23         ' Connection Refused
   Global Const EPhidget_ConnReset = &H2E       ' Connection Reset
   Global Const EPhidget_HostUnreach = &H30     ' No Route To Host
   Global Const EPhidget_NoDev = &H28           ' No Such Device
   Global Const EPhidget_WrongDevice = &H32     ' Wrong Device
   Global Const EPhidget_Pipe = &H29            ' Broken Pipe
   Global Const EPhidget_Resolv = &H2C          ' Name Resolution Failure
   Global Const EPhidget_UnknownVal = &H33      ' Unknown Or InValid Value
   Global Const EPhidget_NotAttached = &H34     ' Device Not Attached
   Global Const EPhidget_InValidPacket = &H35   ' InValid Or Unexpected Packet
   Global Const EPhidget_2big = &H36            ' Argument List Too Long
   Global Const EPhidget_BadVersion = &H37      ' Bad Version
   Global Const EPhidget_Closed = &H38          ' Closed
   Global Const EPhidget_NotConfigured = &H39   ' Not Configured
   Global Const EPhidget_Eof = &H1F  ' End Of File

   ' Phidget_ErrorEventCode - The Error Code From An Error Event
   '
   Const EePhidget_BadVersion = &H1       ' Client And Server Protocol Versions Don't Match.
   Const EePhidget_Busy = &H2             ' Phidget Is In Use.
   Const EePhidget_Network = &H3          ' Networking Communication Error.
   Const EePhidget_Dispatch = &H4         ' An Error Occured Dispatching A Command Or Event.
   Const EePhidget_Failure = &H5          ' A General Failure Occured - See Description For Details.
   Const EePhidget_Ok = &H1000            ' An Error State Has Cleared.
   Const EePhidget_Overrun = &H1002       ' A Sampling Overrun Happened In Firmware.
   Const EePhidget_PacketLost = &H1003    ' One Or More Packets Were Lost.
   Const EePhidget_Wrap = &H1004          ' A Variable Has Wrapped Around.
   Const EePhidget_OverTemp = &H1005      ' Over-Temperature Condition Detected.
   Const EePhidget_OverCurrent = &H1006   ' Over-Current Condition Detected.
   Const EePhidget_OutOfRange = &H1007    ' Out Of Range Condition Detected.
   Const EePhidget_BadPower = &H1008      ' Power Supply Problem Detected.
   Const EePhidget_Saturation = &H1009    ' Saturation Condition Detected.
   Const EePhidget_OverVoltage = &H100B   ' Over-Voltage Condition Detected.
   Const EePhidget_Failsafe = &H100C      ' Fail-Safe Condition Detected.
   Const EePhidget_VoltageError = &H100D  ' Voltage Error Detected.
   Const EePhidget_EnergyDump = &H100E    ' Energy Dump Condition Detected.
   Const EePhidget_Motorstall = &H100F    ' Motor Stall Detected.

   ' Phidget_DeviceID
   '
   Const PhidID_Nothing = &H0    ' Unknown Device
   Const PhidID_InterfaceKit_4_8_8 = &H1 ' PhidgetInterfaceKit 4/8/8
   Const PhidID_1000 = &H2       ' PhidgetServo 1-Motor (1000)
   Const PhidID_1001 = &H3       ' PhidgetServo 4-Motor (1001)
   Const PhidID_1002 = &H4       ' PhidgetAnalog 4-Output (1002)
   Const PhidID_1008 = &H5       ' PhidgetAccelerometer 2-Axis (1008)
   Const PhidID_1010_1013_1018_1019 = &H6 ' PhidgetInterfaceKit 8/8/8 (1010, 1013, 1018, 1019)
   Const PhidID_1011 = &H7       ' PhidgetInterfaceKit 2/2/2 (1011)
   Const PhidID_1012 = &H8       ' PhidgetInterfaceKit 0/16/16 (1012)
   Const PhidID_1014 = &H9       ' PhidgetInterfaceKit 0/0/4 (1014)
   Const PhidID_1015 = &HA       ' PhidgetLinearTouch (1015)
   Const PhidID_1016 = &HB       ' PhidgetCircularTouch (1016)
   Const PhidID_1017 = &HC       ' PhidgetInterfaceKit 0/0/8 (1017)
   Const PhidID_1023 = &HD       ' PhidgetRfid (1023)
   Const PhidID_1024 = &HE       ' PhidgetRfid Read-Write (1024)
   Const PhidID_1030 = &HF       ' Phidgetled-64 (1030)
   Const PhidID_1031 = &H10      ' Phidgetled-64 Advanced (1031)
   Const PhidID_1032 = &H11      ' Phidgetled-64 Advanced (1032)
   Const PhidID_1040 = &H12      ' Phidgetgps (1040)
   Const PhidID_1041 = &H13      ' PhidgetSpatial 0/0/3 Basic (1041)
   Const PhidID_1042 = &H14      ' PhidgetSpatial 3/3/3 Basic (1042)
   Const PhidID_1043 = &H15      ' PhidgetSpatial Precision 0/0/3 High Resolution (1043)
   Const PhidID_1044 = &H16      ' PhidgetSpatial Precision 3/3/3 High Resolution (1044)
   Const PhidID_1045 = &H17      ' PhidgetTemperatureSensor Ir (1045)
   Const PhidID_1046 = &H18      ' PhidgetBridge 4-Input (1046)
   Const PhidID_1047 = &H19      ' PhidgetEncoder HighSpeed 4-Input (1047)
   Const PhidID_1048 = &H1A      ' PhidgetTemperatureSensor 4-Input (1048)
   Const PhidID_1049 = &H1B      ' PhidgetSpatial 0/0/3 (1049)
   Const PhidID_1051 = &H1C      ' PhidgetTemperatureSensor 1-Input (1051)
   Const PhidID_1052 = &H1D      ' PhidgetEncoder Mechanical (1052)
   Const PhidID_1053 = &H1E      ' PhidgetAccelerometer 2-Axis (1053)
   Const PhidID_1054 = &H1F      ' PhidgetfrequencyCounter (1054)
   Const PhidID_1055 = &H20      ' Phidgetir (1055)
   Const PhidID_1056 = &H21      ' PhidgetSpatial 3/3/3 (1056)
   Const PhidID_1057 = &H22      ' PhidgetEncoder HighSpeed (1057)
   Const PhidID_1058 = &H23      ' PhidgetPhSensor (1058)
   Const PhidID_1059 = &H24      ' PhidgetAccelerometer 3-Axis (1059)
   Const PhidID_1060 = &H25      ' PhidgetMotorControl Lv (1060)
   Const PhidID_1061 = &H26      ' PhidgetAdvancedServo 8-Motor (1061)
   Const PhidID_1062 = &H27      ' PhidgetStepper Unipolar 4-Motor (1062)
   Const PhidID_1063 = &H28      ' PhidgetStepper Bipolar 1-Motor (1063)
   Const PhidID_1064 = &H29      ' PhidgetMotorControl Hc (1064)
   Const PhidID_1065 = &H2A      ' PhidgetMotorControl 1-Motor (1065)
   Const PhidID_1066 = &H2B      ' PhidgetAdvancedServo 1-Motor (1066)
   Const PhidID_1067 = &H2C      ' PhidgetStepper Bipolar Hc (1067)
   Const PhidID_1202_1203 = &H2D
   Const PhidID_1204 = &H2E      ' PhidgetTextLcd Adapter (1204)
   Const PhidID_1215__1218 = &H2F ' PhidgetTextLcd 20x2 (1215/1216/1217/1218)
   Const PhidID_1219__1222 = &H30
   Const PhidID_Adp1000 = &H31   ' Ph Adapter
   Const PhidID_Daq1000 = &H33   ' Analog Input Module X8
   Const PhidID_Daq1200 = &H34   ' Digital Input 4
   Const PhidID_Daq1300 = &H35   ' Digital Input 4 Isolated
   Const PhidID_Daq1301 = &H36   ' Digital Input 16
   Const PhidID_Daq1400 = &H37   ' Versatile Input
   Const PhidID_Daq1500 = &H38   ' Bridge
   Const PhidID_Dcc1000 = &H39   ' Dc Motor Controller With Pid
   Const PhidID_Dst1000 = &H3A   ' 200mm Distance Sensor
   Const PhidID_Dst1200 = &H3B   ' Sonar Distance Sensor
   Const PhidID_Enc1000 = &H3C   ' Encoder
   Const PhidID_Hin1000 = &H3D   ' Capacitive Touch Sensor
   Const PhidID_Hin1001 = &H3E   ' Capacitive Scroll
   Const PhidID_Hin1100 = &H3F   ' Joystick
   Const PhidID_Hub0000 = &H40   ' Phidget Hub With 6 Ports
   Const PhidID_Hub0004 = &H43   ' Phidget Spi Vint Hub With 6 Ports
   Const PhidID_Hum1000 = &H45   ' Humidity Sensor
   Const PhidID_Lcd1100 = &H46   ' Lcd
   Const PhidID_Led1000 = &H47   ' Led Driver 32
   Const PhidID_Lux1000 = &H48   ' Light Sensor
   Const PhidID_Mot1100 = &H49   ' Accelerometer 0/0/3
   Const PhidID_Mot1101 = &H4A   ' Spatial 3/3/3
   Const PhidID_Out1000 = &H4B   ' Analog Output 0-5v
   Const PhidID_Out1001 = &H4C   ' Analog Output (+/-)10v
   Const PhidID_Out1002 = &H4D   ' Analog Output (+/-)10v - 16 Bit
   Const PhidID_Out1100 = &H4E   ' Digital Output 4
   Const PhidID_Pre1000 = &H4F   ' Barometer
   Const PhidID_Rcc1000 = &H50   ' 8-Servo Controller
   Const PhidID_Rel1000 = &H51   ' Power Relay 4
   Const PhidID_Rel1100 = &H52   ' Digital Output 4 Isolated
   Const PhidID_Rel1101 = &H53   ' Digital Output 16 Isolated
   Const PhidID_Saf1000 = &H54   ' Power Supply Protector
   Const PhidID_Snd1000 = &H55   ' Sound Pressure Level Sensor
   Const PhidID_Stc1000 = &H56   ' Bipolar Stepper Motor Controller
   Const PhidID_Tmp1000 = &H57   ' Integrated Temperature Sensor
   Const PhidID_Tmp1100 = &H58   ' Thermocouple 1
   Const PhidID_Tmp1101 = &H59   ' Thermocouple 4
   Const PhidID_Tmp1200 = &H5A   ' Rtd
   Const PhidID_Vcp1000 = &H5C   ' Voltage Sensor High Precision
   Const PhidID_Vcp1001 = &H5D   ' Voltage Sensor LArge
   Const PhidID_Vcp1002 = &H5E   ' Voltage Sensor Small
   Const PhidID_DigitalInput_Port = &H5F ' Hub Port In Digital Input Mode
   Const PhidID_DigitalOutput_Port = &H60 ' Hub Port In Digital Output Mode
   Const PhidID_VoltageInput_Port = &H61 ' Hub Port In Voltage Input Mode
   Const PhidID_VoltageRatioInput_Port = &H62 ' Hub Port In Voltage Ratio Input Mode
   Const PhidID_Vcp1100 = &H69   ' 30a Current Sensor
   Const PhidID_Dcc1100 = &H6C   ' Bldc Motor Controller
   Const PhidID_Hin1101 = &H6D   ' Dial Encoder
   Const PhidID_Dcc1001 = &H6E   ' Small Dc Motor Controller
   Const PhidID_Dictionary = &H6F ' Dictionary
   Const PhidID_Stc1001 = &H73   ' Bipolar Stepper Motor SmallController Phidg

   ' Phidget Device Class
   '
   Const PhidClass_Accelerometer = &H1       ' Phidget Accelerometer Device
   Const PhidClass_AdvancedServo = &H2       ' Phidget AdvancedServo Device
   Const PhidClass_Analog = &H3              ' Phidget Analog Device
   Const PhidClass_Bridge = &H4              ' Phidget Bridge Device
   Const PhidClass_Encoder = &H5             ' Phidget Encoder Device
   Const PhidClass_FrequencyCounter = &H6    ' Phidget FrequencyCounter Device
   Const PhidClass_Gps = &H7                 ' Phidget Gps Device
   Const PhidClass_Hub = &H8                 ' Phidget Vint Hub Device
   Const PhidClass_InterfaceKit = &H9        ' Phidget InterfaceKit Device
   Const PhidClass_Ir = &HA                  ' Phidget Ir Device
   Const PhidClass_Led = &HB                 ' Phidget Led Device
   Const PhidClass_MotorControl = &HD        ' Phidget MotorControl Device
   Const PhidClass_PhSensor = &HE            ' Phidget PhSensor Device
   Const PhidClass_Rfid = &HF                ' Phidget Rfid Device
   Const PhidClass_Servo = &H10              ' Phidget Servo Device
   Const PhidClass_Spatial = &H11            ' Phidget Spatial Device
   Const PhidClass_Stepper = &H12            ' Phidget Stepper Device
   Const PhidClass_TemperatureSensor = &H13  ' Phidget TemperatureSensor Device
   Const PhidClass_TextLcd = &H14            ' Phidget TextLcd Device
   Const PhidClass_Vint = &H15               ' Phidget Vint Device
   Const PhidClass_Dictionary = &H18         ' Dictionary Device
   
   ' Phidget_ChannelClass
   '
   Const PhidChClass_Accelerometer = &H1     ' Accelerometer Channel
   Const PhidChClass_CurrentInput = &H2      ' Current Input Channel
   Const PhidChClass_DcMotor = &H4           ' Dc Motor Channel
   Const PhidChClass_DigitalInput = &H5      ' Digital Input Channel
   Const PhidChClass_DigitalOutput = &H6     ' Digital Output Channel
   Const PhidChClass_DistanceSensor = &H7    ' Distance Sensor Channel
   Const PhidChClass_Encoder = &H8           ' Encoder Channel
   Const PhidChClass_FrequencyCounter = &H9  ' Frequency Counter Channel
   Const PhidChClass_Gps = &HA               ' Gps Channel
   Const PhidChClass_Lcd = &HB               ' Lcd Channel
   Const PhidChClass_Gyroscope = &HC         ' Gyroscope Channel
   Const PhidChClass_Hub = &HD               ' Vint Hub Channel
   Const PhidChClass_CapacitiveTouch = &HE   ' Capacitive Touch Channel
   Const PhidChClass_HumiditySensor = &HF    ' Humidity Sensor Channel
   Const PhidChClass_Ir = &H10               ' Ir Channel
   Const PhidChClass_LightSensor = &H11      ' Light Sensor Channel
   Const PhidChClass_Magnetometer = &H12     ' Magnetometer Channel
   Const PhidChClass_PhSensor = &H25         ' Ph Sensor Channel
   Const PhidChClass_PowerGuard = &H14       ' Power Guard Channel
   Const PhidChClass_PressureSensor = &H15   ' Pressure Sensor Channel
   Const PhidChClass_RcServo = &H16          ' Rc Servo Channel
   Const PhidChClass_ResistanceInput = &H17  ' Resistance Input Channel
   Const PhidChClass_Rfid = &H18             ' Rfid Channel
   Const PhidChClass_SoundSensor = &H19      ' Sound Sensor Channel
   Const PhidChClass_Spatial = &H1A          ' Spatial Channel
   Const PhidChClass_Stepper = &H1B          ' Stepper Channel
   Const PhidChClass_TemperatureSensor = &H1C   ' Temperature Sensor Channel
   Const PhidChClass_VoltageInput = &H1D     ' Voltage Input Channel
   Const PhidChClass_VoltageOutput = &H1E    ' Voltage Output Channel
   Const PhidChClass_VoltageRatioInput = &H1F   ' Voltage Ratio Input Channel
   Const PhidChClass_MotorPositionController = &H22   ' Motor Position Control Channel.
   Const PhidChClass_BldcMotor = &H23        ' Bldc Motor Channel
   Const PhidChClass_Dictionary = &H24       ' Dictionary

   ' Phidget Channel Sub Class
   '
   Const PhidchSubClass_None = &H1  ' No SubClass
   Const PhidchSubClass_DigitalOutput_Duty_Cycle = &H10  ' Digital Output Duty Cycle
   Const PhidchSubClass_DigitalOutput_Led_Driver = &H11  ' Digital Output Led Driver
   Const PhidchSubClass_TemperatureSensor_Rtd = &H20     ' Temperature Sensor Rtd
   Const PhidchSubClass_TemperatureSensor_Thermocouple = &H21  ' Temperature Sensor Thermocouple
   Const PhidchSubClass_VoltageInput_Sensor_Port = &H30  ' Voltage Sensor Port
   Const PhidchSubClass_VoltageRatioInput_Sensor_Port = &H40  ' Voltage Ratio Sensor Port
   Const PhidchSubClass_VoltageRatioInput_Bridge = &H41  ' Voltage Ratio Bridge Input
   Const PhidchSubClass_Lcd_GraPhic = &H50               ' GraPhic Lcd
   Const PhidchSubClass_Lcd_Text = &H51                  ' Text Lcd
   Const PhidchSubClass_Encoder_Mode_Settable = &H60     ' Encoder Io Mode Settable
   Const PhidchSubClass_RcServo_Embedded = &H70          ' Rc Servo Embedded
   
   ' Constants:
   '
   Global Const Phidget_SerialNumber_Any = -1
   '     Pass to Phidget_setDeviceSerialNumber() to open any serial number.
   Global Const Phidget_HubPort_Any = -1
   '     Pass to Phidget_setHubPort() to open any hub port.
   Global Const Phidget_Channel_Any = -1
   '      Pass to Phidget_setChannel() to open any channel.
   Global Const Phidget_Label_Any = Null
   '     NULL Pass to Phidget_setDeviceLabel() to open any label.
   Global Const Phidget_Timeout_Infinite = 0
   '     Pass to Phidget_openWaitForAttachment() for an infinite timeout.
   Global Const Phidget_Timeout_Default = 500
   '     Pass to Phidget_openWaitForAttachment() for the default timeout.
   '
   '
 
   ' =======================================================================
   Public Declare Sub Sleep _
      Lib "kernel32" ( _
      ByVal dwMilliseconds&)
   
   Public Declare Function StrLen& _
      Lib "kernel32.dll" _
      Alias "lstrlenA" ( _
      ByVal strPtr&)

   Public Declare Function SysAllocStringByteLen$ _
      Lib "oleaut32.dll" ( _
      Optional ByVal pszStrPtr&, _
      Optional ByVal strLength&)
  



' ===========================================================================
' PHIDGET COMMON MODULE FUNCTIONS
' ===========================================================================
' Contains various functions such as Release, Retain, and getParent designed
' to be used with the Phidget Manager.  These specialized functions may be
' safely ignored if your application does not require a Manager.
   
   ' =======================================================================
   ' Functions:
   ' =======================================================================

   ' =======================================================================
   ' Phidget_getErrorDescription(
   '     PhidgetReturnCode errorCode,
   '     const char **errorString
   ' )
   ' Prameters:
   '     errorCode: The error code
   '     errorString: The error string
   ' Return: The Phidget return code
   ' Gets the string describing the given Phidget return code
   ' NOTE: To get the error String from the error Code, use
   '       errStr$ = PhGetErrorDescription&(errCode&)
   '       in place of this function.
   ' OR:   After calling this function use:
   '       ErrStr$ = SysAllocStringByteLen$(errStrPtr&, StrLen&(errStrPtr&))
   '
   Public Declare Function Phidget_getErrorDescription& _
      Lib "Phidget22.dll" ( _
      ByVal errCode&, _
      ByRef errStrPtr&)
      
   ' =======================================================================
   ' Phidget_finalize(
   '     int32_t  flags
   ' )
   ' Prameters:
   '     flags: Reserved for future use. Pass 0.
   ' Return: The Phidget return code
   ' Release memory and threads used by the Phidget library.  Should be
   ' called prior to unloading the library from the address space.
   ' This function is intended for use in special cases where it is
   ' desired for the Phidget library to be unloaded before a program's
   ' termination.
   '
   Public Declare Function PhFinalize& _
      Lib "Phidget22.dll" _
      Alias "Phidget_finalize" ( _
      ByVal flags&)           ' unsigned integer

   ' =======================================================================
   ' Phidget_getLibraryVersion(
   '     const char **libraryVersion
   ' )
   ' Prameters:
   '     libraryVersion: The Phidget library version.
   ' Return: The Phidget return code
   '
   ' Gets the version of the Phidget library being used by the program.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
    Public Declare Function PhGetLibraryVersion& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getLibraryVersion" ( _
      ByRef libraryVersionStrPtr&)

   ' =======================================================================
   ' Phidget_release(
   '     PhidgetHandle *phid
   ' )
   ' Prameters:
   '     phid: The Phidget handle to release
   ' Return: The Phidget return code
   ' Releases an existing reference to a Phidget handle. This may cause the
   ' handle's resources to be freed; therefore, the handle should not be
   ' referred to again after it is released.  Phidget_release() must not be
   ' called on handles that have not had Phidget_retain() called on them.
   '
   Public Declare Function PhRelease& _
      Lib "Phidget22.dll" _
      Alias "Phidget_release" ( _
      ByRef handle&)

   ' =======================================================================
   ' Phidget_retain(
   '     PhidgetHandle phid
   ' )
   ' Prameters:
   '     phid: The Phidget handle to retain
   ' Return: The Phidget return code
   ' Creates a additional reference to a Phidget handle. Phidget_retain()
   ' is used to keep a handle from a Manager event alive after the event
   ' method returns.
   '
   Public Declare Function PhRetain& _
      Lib "Phidget22.dll" _
      Alias "Phidget_retain" ( _
      ByVal handle&)
   
   ' =======================================================================
   ' Phidget_getAttached(
   '     PhidgetHandle ch,
   '     int *attached
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     attached: True if the channel is attached
   ' Return: The Phidget return code
   ' Gets the attached status of channel. A Phidget is attached after it
   ' has been opened and the Phidget library finds and connects to the
   ' corresponding hardware device.  Most API calls are only valid on
   ' attached Phidgets.
   '
   Public Declare Function PhGetAttached& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getAttached" ( _
      ByVal handle&, _
      ByRef attached&)

   ' =======================================================================
   ' Phidget_getChannel(
   '     PhidgetHandle ch,
   '     int *channel
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     channel: The channel index
   ' Return: The Phidget return code
   ' Gets the channel index of the channel on the device.
   '
   Public Declare Function PhGetChannel& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getChannel" ( _
      ByVal handle&, _
      ByRef channel&)

   ' =======================================================================
   ' Phidget_setChannel(
   '     PhidgetHandle ch,
   '     int channel
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     channel: The channel index
   ' Return: The Phidget return code
   ' Specifies the channel index to be opened. The default channel is 0.
   ' Set to PHIDGET_CHANNEL_ANY to open any channel on the specified device.
   '
   Public Declare Function PhSetChannel& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setChannel" ( _
      ByVal handle&, _
      ByVal channel&)

   ' =======================================================================
   ' Phidget_getChannelClass(
   '     PhidgetHandle ch,
   '     Phidget_ChannelClass *channelClass
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     channelClass: The channel class
   ' Return: The Phidget return code
   ' Gets the channel class of the channel.
   Public Declare Function PhGetChannelClass& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getChannelClass" ( _
      ByVal handle&, _
      ByRef channelClass&)

   ' =======================================================================
   ' Phidget_getChannelClassName(
   '     PhidgetHandle ch,
   '     const char **channelClassName
   ' )
   ' Prameters:
   ' Gets the name of the channel class the channel belongs to.
   '     ch: The channel handle
   '     channelClassName: The name of the channel's class
   ' Return: The Phidget return code
   '     Constant: PHIDCHCLASS_HUB
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
   Public Declare Function PhGetChannelClassName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getChannelClassName" ( _
      ByVal handle&, _
      ByRef serverHostnameStrPtr&)

   ' =======================================================================
   ' Phidget_getChannelName(
   '     PhidgetHandle ch,
   '     const char **channelName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     channelName: The channel's name
   ' Return: The Phidget return code
   ' Gets the channel's name. This name serves as a description of the
   ' specific nature of the channel.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
   Public Declare Function PhGetChannelName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getChannelName" ( _
      ByVal handle&, _
      ByRef channelNameStrPtr&)

   ' =======================================================================
   ' Phidget_getChannelSubclass(
   '     PhidgetHandle ch,
   '     Phidget_ChannelSubclass *channelSubclass
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     channelSubclass: The channel's subclass
   ' Return: The Phidget return code
   ' Gets the subclass for this channel. Allows for identifying channels
   ' with specific characteristics without needing to know the exact
   ' device and channel index.
   '
   Public Declare Function PhGetChannelSubclass& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getChannelSubclass" ( _
      ByVal handle&, _
      ByRef channelSubclass&)

   ' =======================================================================
   ' Phidget_close(
   '     PhidgetHandle ch
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     Constant: PHIDCHSUBCLASS_NONE
   ' Return: The Phidget return code
   ' Closes a Phidget channel that has been opened. Phidget_close() will
   ' release the channel on the Phidget device, and should be called
   ' prior to delete.
   '
   Public Declare Function PhClose& _
      Lib "Phidget22.dll" _
      Alias "Phidget_close" ( _
      ByVal handle&)

   ' =======================================================================
   ' Phidget_getDataInterval(
   '     PhidgetHandle ch,
   '     uint32_t *dataInterval
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     dataInterval: The data interval of the channel
   ' Return: The Phidget return code
   ' Gets the data interval for any channel that supports one.
   ' The DataInterval is the time that must elapse before the channel
   ' will fire another data event.  The data interval is bounded by
   ' MinDataInterval and MaxDataInterval.  The timing between data events
   ' can also be affected by the specific change trigger for the channel.
   '
   Public Declare Function PhGetDataInterval& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDataInterval" ( _
      ByVal handle&, _
      ByRef dataInterval&)    ' unsigned integer
      
   ' =======================================================================
   ' Phidget_setDataInterval(
   '     PhidgetHandle ch,
   '     uint32_t dataInterval
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     dataInterval: The data interval of the channel
   ' Return: The Phidget return code
   ' Sets the data interval for any channel that supports one.
   ' The DataInterval is the time that must elapse before the channel
   ' will fire another data event.  The data interval is bounded by
   ' MinDataInterval and MaxDataInterval.  The timing between data events
   ' can also be affected by the specific change trigger for the channel.
   '
   Public Declare Function PhSetDataInterval& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setDataInterval" ( _
      ByVal handle&, _
      ByVal dataInterval&)    ' unsigned integer
   
   ' =======================================================================
   ' Phidget_getDeviceChannelCount(
   '     PhidgetHandle ch,
   '     Phidget_ChannelClass cls,
   '     uint32_t *count
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     cls: The Channel Class
   '     count: The Channel Count
   ' Return: The Phidget return code
   ' Gets the number of channels of the specified channel class on the
   ' device.  Pass PHIDCHCLASS_NOTHING to get the total number of channels.
   '
   Public Declare Function PhGetDeviceChannelCount& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceChannelCount" ( _
      ByVal handle&, _
      ByRef count&)           ' unsigned integer

   ' =======================================================================
   ' Phidget_getDeviceClass(
   '     PhidgetHandle ch,
   '     Phidget_DeviceClass *deviceClass
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceClass: The class of the device the channel is a part of.
   ' Return: The Phidget return code
   ' Gets the device class for the Phidget which this channel is a part of.
   '
   Public Declare Function PhGetDeviceClass& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceClass" ( _
      ByVal handle&, _
      ByRef deviceClass&)

   ' =======================================================================
   ' Phidget_getDeviceClassName(
   '     PhidgetHandle ch,
   '     const char **deviceClassName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceClassName: The class name of the device the channel is a part of.
   ' Return: The Phidget return code
   '     Constant: PHIDCLASS_HUB
   ' Gets the name of the device class for the Phidget which this channel
   ' is a part of.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
   Public Declare Function PhGetDeviceClassName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceClassName" ( _
      ByVal handle&, _
      ByRef deviceClassNameStrPtr&)

   ' =======================================================================
   ' Phidget_getDeviceID(
   '     PhidgetHandle ch,
   '     Phidget_DeviceID *deviceID
   ' )
   ' Prameters:
   ' Gets the DeviceID for the Phidget which this channel is a part of.
   '     ch: The channel handle
   '     deviceID: The device id of the device the channel is a part of
   ' Return: The Phidget return code
   '
   Public Declare Function PhGetDeviceID& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceID" ( _
      ByVal handle&, _
      ByRef deviceId&)

   ' =======================================================================
   ' Phidget_getDeviceLabel(
   '     PhidgetHandle ch,
   '     const char **deviceLabel
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceLabel: The device label
   ' Return: The Phidget return code
   ' Gets the label of the Phidget which this channel is a part of. A
   ' device label is a custom string used to more easily identify a
   ' Phidget. Labels are written to a Phidget using Phidget_writeDeviceLabel(),
   ' or by right-clicking the device and setting a label in the Phidget Control
   ' Panel for Windows.
   ' See using a label for more information about how to use labels with Phidgets.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(deviceLabelStrPtr&, StrLen&(deviceLabelStrPtr&))
   '
   Public Declare Function PhGetDeviceLabel& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceLabel" ( _
      ByVal handle&, _
      ByRef deviceLabelStrPtr&)

   ' =======================================================================
   ' Phidget_setDeviceLabel(
   '     PhidgetHandle ch,
   '     const char *deviceLabel
   ' )
   ' Prameters:
   '     Constant: PhidID_HUB0000
   '     ch: The channel handle
   '     DeviceLabel: The device label
   ' Return: The Phidget return code
   ' Specifies the label of the Phidget to be opened.  Leave un-set to open
   ' any label.  A device label is a custom string used to more easily
   ' identify a Phidget.  Labels are written to a Phidget using
   ' Phidget_writeDeviceLabel(), or by right-clicking the device
   ' and setting a label in the Phidget Control Panel for Windows.
   ' See using a label for more information about how to use labels with Phidgets.
   ' Example:  stat& = PhSetDeviceLabel&(handle&, StrPtr(deviceLabel$))
   '
   ' Note! This writes data to the Flash memory that has a limited number
   '       of writes and is not supported on very old Phidgets.
   '       I'm not sure it works with VB6.
   '
   Public Declare Function PhSetDeviceLabel& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setDeviceLabel" ( _
      ByVal handle&, _
      ByRef deviceLabel$)
   
   ' =======================================================================
   ' Phidget_getDeviceName(
   '     PhidgetHandle ch,
   '     const char **deviceName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceName: The name of the device the channel is a part of
   ' Return: The Phidget return code
   ' Gets the name of the Phidget which this channel is a part of.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
   Public Declare Function PhGetDeviceName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceName" ( _
      ByVal handle&, _
      ByRef deviceNameStrPtr&)

   ' =======================================================================
   ' Phidget_getDeviceSerialNumber(
   '     PhidgetHandle ch,
   '     int32_t *deviceSerialNumber
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceSerialNumber: The device serial number
   ' Return: The Phidget return code
   ' Gets the serial number of the Phidget which this channel is a part of.
   ' If the channel is part of a VINT device, this will be the serial
   ' number of the VINT Hub the device is attached to.
   '
   Public Declare Function PhGetDeviceSerialNumber& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceSerialNumber" ( _
      ByVal handle&, _
      ByRef deviceSerialNumber&)

   ' =======================================================================
   ' Phidget_setDeviceSerialNumber(
   '     PhidgetHandle ch,
   '     int32_t deviceSerialNumber
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceSerialNumber: The device serial number
   ' Return: The Phidget return code
   ' Specifies the serial number of the Phidget to be opened. Leave un-set,
   ' or set to PHIDGET_SERIALNUMBER_ANY to open any serial number.  If the
   ' channel is part of a VINT device, this will be the serial number of
   ' the VINT Hub the device is attached to.
   '
   Public Declare Function PhSetDeviceSerialNumber& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setDeviceSerialNumber" ( _
      ByVal handle&, _
      ByVal deviceSerialNumber&)

   ' =======================================================================
   ' Phidget_getDeviceSKU(
   '     PhidgetHandle ch,
   '     const char **deviceSKU
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceSKU: The SKU of the device the channel is a part of
   ' Return: The Phidget return code
   ' Gets the SKU (part number) of the Phidget which this channel is a part of.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
   Public Declare Function PhGetDeviceSKU& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceSKU" ( _
      ByVal handle&, _
      ByRef deviceSkuStrPtr&)

   ' =======================================================================
   ' Phidget_getDeviceVersion(
   '     PhidgetHandle ch,
   '     int *deviceVersion
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceVersion: The version of the device the channel is a part of
   ' Return: The Phidget return code
   ' Gets the firmware version of the Phidget which this channel is a part of.
   '
   Public Declare Function PhGetDeviceVersion& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getDeviceVersion" ( _
      ByVal handle&, _
      ByRef deviceVersion&)

   ' =======================================================================
   ' Phidget_getHub(
   '     PhidgetHandle ch,
   '     PhidgetHandle *hub
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     hub: The hub the channels device is attached to
   ' Return: The Phidget return code
   ' Gets the hub that this channel is attached to. Must be freed with
   ' Phidget_release()
   '
   Public Declare Function PhGetHub& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getHub" ( _
      ByVal handle&, _
      ByRef hub&)

   ' =======================================================================
   ' Phidget_getHubPort(
   '     PhidgetHandle ch,
   '     int *hubPort
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     hubPort: The hub port index
   ' Return: The Phidget return code
   ' Gets the hub port index of the VINT Hub port that the channel is attached to.
   '
   Public Declare Function PhGetHubPort& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getHubPort" ( _
      ByVal handle&, _
      ByRef hubPort&)

   ' =======================================================================
   ' Phidget_setHubPort(
   '     PhidgetHandle ch,
   '     int hubPort
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     hubPort: The hub port index
   ' Return: The Phidget return code
   ' Specifies the hub port index of the VINT Hub port to open this channel
   ' on.  Leave un-set, or set to PHIDGET_HUBPORT_ANY to open the channel on
   ' any VINT Hub port
   Public Declare Function PhSetHubPort& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setHubPort" ( _
      ByVal handle&, _
      ByVal hubPort&)

   ' =======================================================================
   ' Phidget_getHubPortCount(
   '     PhidgetHandle ch,
   '     int *hubPortCount
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     hubPortCount: The number of ports on the VINT Hub
   ' Return: The Phidget return code
   ' Gets the number of VINT ports present on the VINT Hub that the channel
   ' is attached to.
   '
   Public Declare Function PhGetHubPortCount& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getHubPortCount" ( _
      ByVal handle&, _
      ByRef hubPortCount&)

   ' =======================================================================
   ' Phidget_getIsChannel(
   '     PhidgetHandle ch,
   '     int *isChannel
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isChannel: True if the handle is for a channel.
   ' Return: The Phidget return code
   ' Returns true if the PhidgetHandle is for a channel. Mostly for use
   ' alongside Phidget_getParent() to distinguish channel handles.
   ' from device handles.
   '
   Public Declare Function PhGetIsChannel& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getIsChannel" ( _
      ByVal handle&, _
      ByRef isChannel&)

   ' =======================================================================
   ' Phidget_getIsHubPortDevice(
   '     PhidgetHandle ch,
   '     int *isHubPortDevice
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isHubPortDevice: The hub port mode (True if the channel is a hub port itself)
   ' Return: The Phidget return code
   ' Gets whether this channel is a VINT Hub port channel, or part of a VINT
   ' device attached to a hub port.
   '
   Public Declare Function PhGetIsHubPortDevice& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getIsHubPortDevice" ( _
      ByVal handle&, _
      ByRef isHubPortDevice&)

   ' =======================================================================
   ' Phidget_setIsHubPortDevice(
   '     PhidgetHandle ch,
   '     int isHubPortDevice
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isHubPortDevice: The hub port mode (True if channel is a hub port itself)
   ' Return: The Phidget return code
   ' Specifies whether this channel should be opened on a VINT Hub port
   ' directly, or on a VINT device attached to a hub port.
   '
   Public Declare Function PhSetIsHubPortDevice& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setIsHubPortDevice" ( _
      ByVal handle&, _
      ByVal isHubPortDevice&)

   ' =======================================================================
   ' Phidget_getIsLocal(
   '     PhidgetHandle ch,
   '     int *isLocal
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isLocal: True if the channel is attached to a local device
   ' Return: The Phidget return code
   ' Returns true when this channel is attached directly on the local
   ' machine, or false otherwise.
   '
   Public Declare Function PhGetIsLocal& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getIsLocal" ( _
      ByVal handle&, _
      ByRef isLocal&)

   ' =======================================================================
   ' Phidget_setIsLocal(
   '     PhidgetHandle ch,
   '     int isLocal
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isLocal: True if the channel is attached to a local device
   ' Return: The Phidget return code
   ' Set to True if the channel is to be opened locally, and not over a
   ' network.  If both this and IsRemote are set to False (the default),
   ' the channel will be opened either locally or remotely, on whichever
   ' matching channel is found first.
   '
   Public Declare Function PhSetIsLocal& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setIsLocal" ( _
      ByVal handle&, _
      ByVal isLocal&)

   ' =======================================================================
   ' Phidget_getIsRemote(
   '     PhidgetHandle ch,
   '     int *isRemote
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isRemote: True if the channel is attached to a network device
   ' Return: The Phidget return code
   ' Returns true when this channel is attached via a Phidget network
   ' server, or false otherwise.
   '
   Public Declare Function PhGetIsRemote& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getIsRemote" ( _
      ByVal handle&, _
      ByRef isRemote&)

   ' =======================================================================
   ' Phidget_setIsRemote(
   '     PhidgetHandle ch,
   '     int isRemote
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     isRemote: True if the channel is attached to a network device
   ' Return: The Phidget return code
   ' Set to True if the channel is to be opened remotely, rather than
   ' locally.  If both this and IsLocal are set to False (the default),
   ' the channel will be opened either locally or remotely, on whichever
   ' matching channel is found first.
   '
   Public Declare Function PhSetisRemote& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setIsRemote" ( _
      ByVal handle&, _
      ByVal isRemote&)

   ' =======================================================================
   ' Phidget_open(
   '     PhidgetHandle ch
   ' )
   ' Prameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   ' Opens the Phidget channel. The specific channel to be opened can be
   ' specified by setting any of the following properties:
   '     DeviceSerialNumber
   '     DeviceLabel
   '     Channel
   '     HubPort
   '     IsHubPortDevice
   '     ServerName
   '     IsLocal
   '     IsRemote
   ' Phidget_open() will return immediately, with the attachment process
   ' proceeding asynchronously.  Use the Attach event or Attached property
   ' to determine when the channel is ready to use.
   '
   Public Declare Function PhOpen& _
      Lib "Phidget22.dll" _
      Alias "Phidget_open" ( _
      ByVal handle&)

   ' =======================================================================
   ' Phidget_openWaitForAttachment(
   '     PhidgetHandle ch,
   '     uint32_t  timeout
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     timeout: Timeout in milliseconds
   ' Return: The Phidget return code
   ' Opens the Phidget channel and waits a defined amount of time for
   ' the device to attach.  The specific channel to be opened can be
   ' specified by setting any of the following properties:
   '     DeviceSerialNumber
   '     DeviceLabel
   '     Channel
   '     HubPort
   '     IsHubPortDevice
   '     ServerName
   '     IsLocal
   '     IsRemote
   ' Phidget_openWaitForAttachment() will block until the channel is attached
   ' or a timeout occurs.  A timeout value of 0 will wait forever.
   '
   Public Declare Function PhOpenWaitForAttachment& _
      Lib "Phidget22.dll" _
      Alias "Phidget_openWaitForAttachment" ( _
      ByVal handle&, _
      ByVal timeOutMs&)    ' unsigned integer

   ' =======================================================================
   ' Phidget_getParent(
   '     PhidgetHandle ch,
   '     PhidgetHandle *parent
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     parent: The handle of the parent
   ' Return: The Phidget return code
   ' Gets the handle of the parent device of the given Phidget handle.
   ' For example, this would refer to the device the channel is a part of,
   ' or the Hub that a device is plugged into.  This is useful when used
   ' alongside a Phidget Manager to create device trees like the one in
   ' the Phidget Control Panel. Phidget_release() must be called on the
   ' handle returned by this function to release the reference created
   ' by this call.  This can be used to travel up the device tree and
   ' get device information at each step.  The root device will return
   ' a null handle.  Parent handles always refer to devices.
   ' See Phidget_getIsChannel()
   '
   Public Declare Function PhGetParent& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getParent" ( _
      ByVal handle&, _
      ByRef parent&)

   ' =======================================================================
   ' Phidget_getServerHostname(
   '     PhidgetHandle ch,
   '     const char **serverHostname
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     serverHostname: The hostname of the channel's server
   ' Return: The Phidget return code
   ' Gets the hostname of the Phidget network server for network attached
   ' Phidgets.  Fails if channel is not connected to a Phidget network server.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   '
   Public Declare Function PhGetServerHostname& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getServerHostname" ( _
      ByVal handle&, _
      ByRef serverHostnameStrPtr&)

   ' =======================================================================
   ' Phidget_getServerName(
   '     PhidgetHandle ch,
   '     const char **serverName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     serverName: The name of the Phidget network server the channel is from
   ' Return: The Phidget return code
   ' Gets the name of the Phidget network server the channel is attached to,
   ' if any.  Fails if the channel is not connected to a Phidget network server.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   Public Declare Function PhGetServerName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getServerName" ( _
      ByVal handle&, _
      ByRef serverNameStrPtr&)

   ' =======================================================================
   ' Phidget_setServerName(
   '     PhidgetHandle ch,
   '     const char *serverName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     serverName: The name of the Phidget network server the channel is from
   ' Return: The Phidget return code
   ' Specifies that this channel will be opened remotely, on a Phidget
   ' network server with this name.
   '
   Public Declare Function PhSetServerName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setServerName" ( _
      ByVal handle&, _
      ByRef serverName$)

   ' =======================================================================
   ' Phidget_getServerPeerName(
   '     PhidgetHandle ch,
   '     const char **serverPeerName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     serverPeerName: The address and port of the channel's server
   ' Return: The Phidget return code
   ' Gets the peer name (address and port) of the Phidget server for
   ' network attached Phidgets, formatted as: address:port
   ' Fails if the channel is not connected to a Phidget network server.
   ' NOTE: After calling this function use:
   '       Str$ = SysAllocStringByteLen$(...StrPtr&, StrLen&(...StrPtr&))
   Public Declare Function PhGetServerPeerName& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getServerPeerName" ( _
      ByVal handle&, _
      ByRef serverPeerNameStrPtr&)

   ' =======================================================================
   ' Phidget_writeDeviceLabel(
   '     PhidgetHandle ch,
   '     const char *deviceLabel
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     deviceLabel: The device label
   ' Return: The Phidget return code
   ' Writes a label to the device in the form of a string in the device
   ' flash memory.  This label can then be used to identify the device,
   ' and will persist across power cycles.  The label can be at most
   ' 10 UTF-16 code units.  Most unicode characters take up a single
   ' code unit, but some, such as emoji, can take several.  Some devices
   ' can not have their labels set from Windows.  For these devices the
   ' label should be set from Linux or macOS.
   ' See using a label for more information about how to use labels with Phidgets.
   ' Example:  stat& = PhWriteDeviceLabel&(handle&, StrPtr(deviceLabel$))
   ' Sets the label of a Phidget. Note that this is not supported on
   '     very old Phidgets, and not supported in Windows7.
   '
   Public Declare Function PhWriteDeviceLabel& _
      Lib "Phidget22.dll" _
      Alias "Phidget_writeDeviceLabel" ( _
      ByVal handle&, _
      ByRef Label$)

   ' =======================================================================
   ' Events:
   ' =======================================================================
   
   ' Phidget_setOnAttachHandler(
   '     PhidgetHandle ch,
   '     Phidget_OnAttachCallback handler,
   '     void *ctx
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     handler: The callback function  (AddressOf PhOnAttach)
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   ' Assigns a handler that will be called when the Attach event occurs
   '
   ' This function requires PhOnAttach& callback procedure
   ' handlerPtr& must be set to AddressOf PhOnAttach
   '
   Public Declare Function PhSetOnAttach& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setOnAttachHandler" ( _
      ByVal handle&, _
      ByVal handlerPtr&, _
      ByVal userData&)
    
   ' =======================================================================
   ' Phidget_setOnDetachHandler(
   '     PhidgetHandle ch,
   '     Phidget_OnDetachCallback handler,
   '     void *ctx
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     handler: The callback function  (AddressOf PhOnDetach)
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   ' Assigns a handler that will be called when the Detach event occurs
   '
   ' This function requires PhOnDetach& callback procedure
   ' handlerPtr& must be set to AddressOf PhOnDetach
   '
   Public Declare Function PhSetOnDetach& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setOnDetachHandler" ( _
      ByVal handle&, _
      ByVal handlerPtr&, _
      ByVal userData&)
  
   ' =======================================================================
   ' Phidget_setOnErrorHandler(
   '     PhidgetHandle ch,
   '     Phidget_OnErrorCallback handler,
   '     void *ctx
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     handler: The callback function  (AddressOf PhOnError)
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   ' Assigns a handler that will be called when the Error event occurs
   '
   ' This function requires PhOnError& callback procedure
   ' handlerPtr& must be set to AddressOf PhOnError
   '
   Public Declare Function PhSetOnError& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setOnErrorHandler" ( _
      ByVal handle&, _
      ByVal handlerPtr&, _
      ByVal userData&)

   ' =======================================================================
   ' Phidget_setOnPropertyChangeHandler(
   '     PhidgetHandle ch,
   '     Phidget_OnPropertyChangeCallback handler,
   '     void *ctx
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     handler: The callback function  (AddressOf PhOnPropertyChange)
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   ' Assigns a handler that will be called when the PropertyChange event occurs
   '
   ' This function requires PhOnPropertyChange& callback procedure
   ' handlerPtr& must be set to AddressOf PhOnPropertyChange
   '
   Public Declare Function PhSetOnPropertyChange& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setOnPropertyChangeHandler" ( _
      ByVal handle&, _
      ByVal handlerPtr&, _
      ByVal userData&)



' ===========================================================================
'/* Additional functions */
' ===========================================================================

   ' =======================================================================
   ' Frees a Phidget handle.
   ' phid - A closed phidget handle.
   ' Phidget_delete (PhidgetHandle *phid);
   '
   Public Declare Function PhDelete& _
      Lib "Phidget22.dll" _
      Alias "Phidget_delete" ( _
      ByRef handle&)

   ' =======================================================================
   ' Phidget_getChildDevices (
   '     PhidgetHandle phid,
   '     PhidgetHandle *arr,
   '     size_t *arrCnt);
   '
   Public Declare Function PhGetChildDevices& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getChildDevices" ( _
      ByVal handle&, _
      ByRef arr&, _
      ByRef arrCnt&)

   ' =======================================================================
   ' Phidget_releaseDevices (
   '     PhidgetHandle *arr,
   '     size_t arrCnt);
   '
   Public Declare Function PhReleaseDevices& _
      Lib "Phidget22.dll" _
      Alias "Phidget_releaseDevices" ( _
      ByRef arr&, _
      ByVal arrCnt&)


' ===========================================================================
'/* Mesh device properties */
' ===========================================================================
   
   ' =======================================================================
   ' Phidget_setMeshMode (
   '     PhidgetHandle phid,
   '     Phidget_MeshMode mode);
   '
   Public Declare Function PhSetMeshMode& _
      Lib "Phidget22.dll" _
      Alias "Phidget_setMeshMode" ( _
      ByVal handle&, _
      ByVal mode&)

   ' =======================================================================
   ' Phidget_getMeshMode (
   '     PhidgetHandle phid,
   '     Phidget_MeshMode *mode);
   '
   Public Declare Function PhGetMeshMode& _
      Lib "Phidget22.dll" _
      Alias "Phidget_getMeshMode" ( _
      ByVal handle&, _
      ByRef mode&)


' ===========================================================================
' Event Callbacks, Subs, and Functions
' ===========================================================================

' ===========================================================================
Public Static Sub PhOnAttach(ByVal handle&, ByVal userData&)
   ' Phidget_OnAttachCallback void CCONV onAttach(
   '     PhidgetHandle ch,
   '     void *ctx
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     ctx: Pointer to the user context
   ' Occurs when the channel is attached to a physical channel on a Phidget.
   ' PhSetOnAttach must be called prior to calling Phidget_open(), and
   ' will be called when the Phidget library matches the channel with a
   ' physical channel on a Phidget.  PhOnAttach may be called more than
   ' once if the channel is detached during its lifetime.  PhOnAttach is
   ' the recommended place to configure properties of the channel such
   ' as the data interval or change trigger.
   '
   ' Sample code for startup:
   '     stat& = PhSetOnAttach&(handle&, AddressOf PhOnAttach, userData&)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   Form1.Label6(userData&) = "Attached"
   ' ===================================

End Sub
   
' ===========================================================================
Public Static Sub PhOnDetach(ByVal handle&, ByVal userData&)
   ' Phidget_OnDetachCallback void CCONV onDetach(
   '     PhidgetHandle ch,
   '     void *ctx
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     ctx: Pointer to the user context
   ' Occurs when the channel is detached from a Phidget device channel.
   ' Phidget_OnDetachCallback typically occurs when
   ' the Phidget device is removed from the system.
   '
   ' Sample code for startup:
   '     stat& = PhSetOnDetach&(handle&, AddressOf PhOnDetach, userData&)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' Note:
   ' If I put to much code here, the program will crash on exit when
   ' running in interpeted mode after un-plugging and re-plugging the
   ' USB cable.  EXE still shuts down OK if I don't do any Calls.  ???
   Form1.Label6(userData&) = "Detached"
   Form1.Label7(userData&) = "---"
   ' ===================================
   
   
End Sub
   
' ===========================================================================
Public Static Sub PhOnError(ByVal handle&, ByVal userData&, _
                            ByVal errCode&, ByRef description$)
   ' Phidget_OnErrorCallback void CCONV onError(
   '     PhidgetHandle ch,
   '     void *ctx,
   '     Phidget_ErrorEventCode code,
   '     const char *description
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     ctx: Pointer to the user context
   '     code: The error code
   '     description: The error description
   ' Phidget_OnErrorCallback is called when an error condition has been detected.
   ' See the documentation for your specific channel class to see what error
   ' events it might throw.
   '
   ' Sample code for startup:
   '     stat& = PhSetOnError&(handle&, AddressOf PhOnError, userData&)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   Call Form1.DisplayStatus(userData&, errCode&, description$)
   ' ===================================

End Sub
   
' ===========================================================================
Public Static Sub PhOnPropertyChange(ByVal handle&, ByVal userData&, _
                              ByRef propertyName$)
   ' Phidget_OnPropertyChangeCallback void CCONV onPropertyChange(
   '     PhidgetHandle ch,
   '     void *ctx,
   '     const char *propertyName
   ' )
   ' Prameters:
   '     ch: The channel handle
   '     ctx: Pointer to the user context
   '     propertyName: The name of the property that has changed
   ' Occurs when a property is changed externally from the user channel,
   ' usually from a network client attached to the same channel.
   '
   ' Sample code for startup:
   '     stat& = PhSetPropertyChange&(handle&, _
   '                AddressOf PhOnPropertyChange, userData&)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================
   
End Sub
   
' ===========================================================================
Public Static Function PhGetErrorDescription$(errCode&)
   ' Example:  errStr$ = PhGetErrorDescription&(errCode&)
   '
   ' For returning string data from the Phidget22 DLL.
   ' VB6 expects string data to be returned ByRef.  Which is a
   ' pointer to the actual string.  The Phidget22 DLL returns
   ' a pointer to a pointer which points to the actual data.
   ' The string is also NULL (0) terminated.  The "0" must be
   ' removed before VB can display the string correctly.
   ' Fortunately; There is an API function that takes care of
   ' all of this for us. "SysAllocStringByteLen"

   Dim stat&
   Dim cStrPtr&
   
   PhGetErrorDescription$ = "" ' Return with nothing if error
   stat& = Phidget_getErrorDescription&(errCode&, cStrPtr&) ' Get cStrPtr&
   If stat& = 0 Then
      ' Got pointer to string, now get string
      PhGetErrorDescription$ = SysAllocStringByteLen$(cStrPtr&, StrLen&(cStrPtr&))
   End If
   
End Function

   
   
