Attribute VB_Name = "PhVintDigOut"
Option Explicit

   ' ========================================================================
   ' PhidgetDigitalOutput  OUT1100  (4 digital outputs 30Vdc 8A)
   '
   ' The Digital Output class is used to control digital logic outputs and
   ' LED outputs on Phidgets boards.

   ' ========================================================================
   ' PhidgetDigitalOutput_create(
   '     PhidgetDigitalOutputHandle *ch
   ' )
   ' Parameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   '
   ' Creates an instance of a Phidget channel.  Once created the channel may
   ' need to be configured to match the physical device and channel index you
   ' want to attach to, and then be opened by calling Phidget_open() .
   ' The channel handle must be released with PhidgetDigitalOutput_delete() .
   '
   Public Declare Function PhDigOutCreate& _
      Lib "Phidget22.dll" _
      Alias "PhidgetDigitalOutput_create" ( _
      ByRef handle&)

   ' ========================================================================
   ' PhidgetDigitalOutput_delete(
   '     PhidgetDigitalOutputHandle *ch
   ' )
   ' Parameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   '
   ' Releases a Phidget channel handle created with PhidgetDigitalOutput_create()
   ' If the channel is open when called, it will be closed.
   '
   Public Declare Function PhDigOutDelete& _
      Lib "Phidget22.dll" _
      Alias "PhidgetDigitalOutput_delete" ( _
      ByRef handle&)

   ' ========================================================================
   ' PhidgetDigitalOutput_getDutyCycle(
   '     PhidgetDigitalOutputHandle ch,
   '     double *dutyCycle
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     dutyCycle: Long value pointer to the duty cycle value (double#)
   ' Return: The Phidget return code
   ' Default: 0.0 duty cycle
   '
   ' The DutyCycle represents the fraction of time the output is on (high).
   ' A DutyCycle of 1.0 translates to a high output, a DutyCycle of 0 translates
   ' to a low output.  A DutyCycle of 0.5 translates to an output that is high
   ' half the time, which results in an average output voltage of (output
   ' voltage x 0.5) You can use the DutyCycle to create a dimming effect on LEDs.
   '
   Public Declare Function PhDigOutGetDutyCycle& _
      Lib "Phidget22.dll" _
      Alias "PhidgetDigitalOutput_getDutyCycle" ( _
      ByVal handle&, _
      ByRef dutyCycle#)

   ' ========================================================================
   ' PhidgetDigitalOutput_setDutyCycle(
   '     PhidgetDigitalOutputHandle ch,
   '     double dutyCycle
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     dutyCycle: The duty cycle value
   ' Return: The Phidget return code
   ' Default: 0.0# duty cycle
   '
   ' The DutyCycle represents the fraction of time the output is on (high).
   ' This will override the State setting on the channel. A DutyCycle of 1.0
   ' translates to a high output, a DutyCycle of 0 translates to a low output.
   ' This is equivalent to setting a State of TRUE and FALSE respectively.
   ' A DutyCycle of 0.5 translates to an output that is high half the time,
   ' which results in an average output voltage of (output voltage x 0.5)
   ' You can use the DutyCycle to create a dimming effect on LEDs.
   '
   Public Declare Function PhDigOutSetDutyCycle& _
      Lib "Phidget22.dll" _
      Alias "PhidgetDigitalOutput_setDutyCycle" ( _
      ByVal handle&, _
      ByVal dutyCycle#)

   ' ========================================================================
   ' PhidgetDigitalOutput_setDutyCycle_async(
   '     PhidgetDigitalOutputHandle ch,
   '     double dutyCycle,
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '        void *ctx,
   '        PhidgetReturnCode res),
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     dutyCycle: The duty cycle value
   '     asyncHandler: Pointer to Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' The DutyCycle represents the fraction of time the output is on (high).
   ' This will override the State setting on the channel. A DutyCycle of 1.0
   ' translates to a high output, a DutyCycle of 0 translates to a low output.
   ' This is equivalent to setting a State of TRUE and FALSE respectively. A
   ' DutyCycle of 0.5 translates to an output that is high half the time,
   ' which results in an average output voltage of V * 0.5.  You can use the
   ' DutyCycle to create a dimming effect on LEDs.
   '
   ' This function requires PhDigOutOnSetDutyCycle callback procedure
   ' AsyncHandlerPtr& must be set to AddressOf PhDigOutOnSetDutyCycle
   '
   Public Declare Function PhDigOutSetDutyCycleAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetDigitalOutput_setDutyCycle_async" ( _
      ByVal handle&, _
      ByVal dutyCycle#, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)

   ' ========================================================================
   ' PhidgetDigitalOutput_getMinDutyCycle(
   '     PhidgetDigitalOutputHandle ch,
   '     double *minDutyCycle
   ' )
   ' Parameters:
   '     PhidgetDigitalOutputHandle ch: The channel handle
   '     minDutyCycle: The duty cycle value
   ' Return: The Phidget return code
   '     Constant: Pointer to 0.0# duty cycle
   '
   ' The minimum value that DutyCycle can be set to.
   '
   Public Declare Function PhDigOutGetMinDutyCycle& _
      Lib "Phidget22.dll" _
      Alias "PhidgetDigitalOutput_getMinDutyCycle" ( _
      ByVal handle&, _
      ByRef minDutyCycle#)

   ' ========================================================================
   ' PhidgetDigitalOutput_getMaxDutyCycle(
   '     PhidgetDigitalOutputHandle ch,
   '     double *maxDutyCycle
   ' )
   ' Parameters:
   '     PhidgetDigitalOutputHandle ch: The channel handle
   '     maxDutyCycle: The duty cycle value
   ' Return: The Phidget return code
   '     Constant: Pointer to 0.0# duty cycle
   '
   ' The maximum value that DutyCycle can be set to.
   '
   Public Declare Function PhDigOutGetMaxDutyCycle& _
      Lib "Phidget22.dll" _
      Alias "PhidgetDigitalOutput_getMaxDutyCycle" ( _
      ByVal handle&, _
      ByRef maxDutyCycle#)

   ' ========================================================================
   ' PhidgetDigitalOutput_getState(
   '     PhidgetDigitalOutputHandle ch,
   '     int *state
   ' )
   ' Parameters:
   '     PhidgetDigitalOutputHandle ch: The channel handle
   '     state: Pointer to the state value (16bit true false)
   ' Return: The Phidget return code
   ' Default: 0
   '
   ' The State will indicate whether the output is high (TRUE) or low (FALSE).
   ' If a DutyCycle has been set, the state will return as TRUE if the
   ' DutyCycle is above 0.5, or FALSE otherwise.
   '
   ' Note: This appears to be incorrect.
   '       The state& is high when DutyCycle is >0.
   '
   Public Declare Function PhDigOutGetState& _
      Lib "Phidget22.dll" _
      Alias "PhidgetDigitalOutput_getState" ( _
      ByVal handle&, _
      ByRef state&)

   ' ========================================================================
   ' PhidgetDigitalOutput_setState(
   '     PhidgetDigitalOutputHandle ch,
   '     int  state
   ' )
   ' Parameters: ch: The channel handle
   '     state: The state value (16bit)
   ' Return: The Phidget return code
   ' Default: 0
   '
   ' The State will dictate whether the output is high (TRUE) or Low (FALSE).
   ' This will override any DutyCycle that may have been set on the channel.
   ' Setting the State to TRUE is the same as setting DutyCycle to 1.0, and
   ' setting the State to FALSE is the same as setting a DutyCycle of 0.0.
   '
   Public Declare Function PhDigOutSetState& _
      Lib "Phidget22.dll" _
      Alias "PhidgetDigitalOutput_setState" ( _
      ByVal handle&, _
      ByVal state&)

   ' ========================================================================
   ' PhidgetDigitalOutput_setState_async(
   '     PhidgetDigitalOutputHandle ch,
   '     int  state,
   '     void (CCONV *asyncHandler)
   '       (PhidgetHandle phid,
   '       void *ctxPtr,
   '       PhidgetReturnCode res),
   '     void *ctxtr
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     state: The state value
   '     asyncHandler: Asynchronous completion callback address
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   ' Default: 0
   '
   ' The State will dictate whether the output is constantly high (TRUE) or Low
   ' (FALSE). This will override any DutyCycle that may have been set on the
   ' channel. Setting the State to TRUE is the same as setting DutyCycle to 1.0,
   ' and setting the State to FALSE is the same as setting a DutyCycle of 0.0.
   '
   ' This function requires PhDigOutOnSetState callback procedure
   ' AsyncHandlerPtr& must be set to AddressOf PhDigOutOnSetState
   '
   Public Declare Function PhidgetDigitalOutput_setState_async _
      Lib "Phidget22.dll" ( _
      ByVal handle&, _
      ByVal state&, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)


' ===========================================================================
' Event Callbacks, Subs, and Functions
' ===========================================================================

' ===========================================================================
Public Static Function PhDigOutOnSetState&(ByVal handle&, ByVal userData&)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhDigOutOnSetState& = 0 ' Return Code

End Function

' ===========================================================================
Public Static Function PhDigOutOnSetDutyCycle&(ByVal handle&, ByVal userData&)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   
   ' Just changing color of label to show activity
   If Form1.Label15.BackColor <> &H80A080 Then
      Form1.Label15.BackColor = &H80A080
   Else
      Form1.Label15.BackColor = &H8080A0
   End If
   Form1.Label15 = "uDat= " & CStr(userData&)
   
   ' ===================================
  
   PhDigOutOnSetDutyCycle& = 0 ' Return Code
  
End Function


' ==========================================================================
' PHIDGET Digital Output STARTUP
' ==========================================================================
Public Static Sub InitPhDigOut(indx&, stat&)
   ' Enter:    ioData        (Dev(indx&))
   ' Return:   Dev(indx&).hndl&  (Dev(indx&).hndl&)
   '           stat&  0=OK
   
   ' Create, configure, and open a DigitalOutput channel.
   ' stat& = 0 if the program exits successfully
   
   On Error Resume Next
   stat& = 0& ' Set default OK
   
   ' ===================================
   ' Create an instance of a Phidget channel.  Once created the
   ' channel may need to be configured to match the physical
   ' device and channel you want to attach to, and then be
   ' opened by calling Phidget_open().  The channel handle must
   ' be released with PhidgetDigitalOutput_delete().
   '
   ' stat& (Error Code) will always be 0 if no Errors
   '
   stat& = PhDigOutCreate&(Dev(indx&).hndl) ' Create new handle
   If stat& <> 0 Then GoTo HandleErr
   
   ' SET SERIAL NUMBER
   stat& = PhSetDeviceSerialNumber(Dev(indx&).hndl, Dev(indx&).hubSn)
   If stat& <> 0 Then GoTo HandleErr

   ' SET ISHUB (0 if hub port is connected to a Phidget module)
   stat& = PhSetIsHubPortDevice(Dev(indx&).hndl, Dev(indx&).isHub)
   If stat& <> 0 Then GoTo HandleErr

   ' SET HUB PORT
   stat& = PhSetHubPort(Dev(indx&).hndl, Dev(indx&).port)
   If stat& <> 0 Then GoTo HandleErr

   ' SET CHANNEL
   stat& = PhSetChannel(Dev(indx&).hndl, Dev(indx&).ch)
   If stat& <> 0 Then GoTo HandleErr
   
   ' Setup the handlers to be run when the device is plugged in,
   ' opened from software, unplugged, closed from software, or
   ' generates an error.
   ' SETUP ATTACH EVENT
   stat& = PhSetOnAttach&(Dev(indx&).hndl, _
               AddressOf PhOnAttach, _
               indx&) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' SETUP DETACH EVENT
   stat& = PhSetOnDetach&(Dev(indx&).hndl, _
               AddressOf PhOnDetach, _
               indx&) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' SETUP ERROR EVENT
   ' Using common Error Handler for all phidget modules.
   stat& = PhSetOnError&(Dev(indx&).hndl, _
               AddressOf PhOnError, _
               indx&) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' OPEN CHANNEL
   'stat& = PhOpen(Dev(indx&).hndl)
   
   ' OPEN CHANNEL & WAIT FOR ATTACH with timeout
   stat& = PhOpenWaitForAttachment(Dev(indx&).hndl, timeOutMs&)
   If stat& <> 0 Then GoTo HandleErr
   
   GoTo ExitSub
HandleErr:
   Call Form1.DisplayStatus(indx&, stat&, "InitPhDigOut")

ExitSub:

End Sub



