Attribute VB_Name = "PhVintLcd"
Option Explicit
   
   ' PhidgetLCD_Font
   Global Const FONT_User1 = &H1              ' User-defined font #1
   Global Const FONT_User2 = &H2              ' User-defined font #2
   Global Const FONT_6x10 = &H3               ' 6px by 10px font
   Global Const FONT_5x8 = &H4                ' 5px by 8px font
   Global Const FONT_6x12 = &H5               ' 6px by 12px font

   ' PhidgetLCD_ScreenSize
   Global Const SCREEN_SIZE_64x128& = &HD     ' 64px by 128px graphic screen

   ' PhidgetLCD_PixelState
   Global Const PIXEL_STATE_OFF = &H0         ' Pixel off state
   Global Const PIXEL_STATE_ON = &H1          ' Pixel on state
   Global Const PIXEL_STATE_INVERT = &H2      ' Invert the pixel state

   ' ========================================================================
   ' LCD1100_0 - Graphic LCD Phidget
   '
   ' The LCD class allows you to control various liquid crystal displays.
   ' It offers control of displayed text as well as screen settings and
   ' custom character creation.

   ' ========================================================================
   ' PhidgetLCD_create(
   '     PhidgetLCDHandle *ch
   ' )
   ' Parameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   '
   ' Creates an instance of a Phidget channel. Once created the channel may
   ' need to be configured to match the physical device and channel index
   ' you want to attach to, and then be opened by calling Phidget_open().
   ' The Phidget channel handle must be released with PhidgetLCD_delete() .
   '
   Public Declare Function PhLcdCreate& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_create" ( _
      ByRef handle&)

   ' ========================================================================
   ' PhidgetLCD_delete(
   '     PhidgetLCDHandle *ch,
   ' )
   ' Parameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   '
   ' Releases a Phidget channel handle created with PhidgetLCD_create()
   ' If the channel is open when called, it will be closed.
   '
   Public Declare Function PhLcdDelete& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_delete" ( _
      ByRef handle&)

   ' ========================================================================
   ' PhidgetLCD_getBacklight(
   '     PhidgetLCDHandle ch,
   '     double *backlight
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     backlight: The backlight value
   ' Return: The Phidget return code
   ' Default: 0.0
   '
   ' The Backlight affects the brightness of the LCD screen.
   ' Backlight is bounded by MinBackLight and MaxBacklight.
   '
   Public Declare Function PhLcdGetBacklight& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getBacklight" ( _
      ByVal handle&, _
      ByRef backlight#)

   ' ========================================================================
   ' PhidgetLCD_setBacklight(
   '     PhidgetLCDHandle ch,
   '     double backlight
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     backlight: The backlight value
   ' Return: The Phidget return code
   ' Default: 0.0
   '
   ' The Backlight affects the brightness of the LCD screen.
   ' Backlight is bounded by MinBackLight and MaxBacklight.
   '
   Public Declare Function PhLcdSetBacklight& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_setBacklight" ( _
      ByVal handle&, _
      ByVal backlight#)

   ' ========================================================================
   ' PhidgetLCD_getMinBacklight(
   '     PhidgetLCDHandle ch,
   '     double *minBacklight
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     minbacklight: The backlight value
   ' Return: The Phidget return code
   ' Constant 0.0
   '
   ' The minimum value that Backlight can be set to.
   '
   Public Declare Function PhLcdGetMinBacklight& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getMinBacklight" ( _
      ByVal handle&, _
      ByRef minBacklight#)

   ' ========================================================================
   ' PhidgetLCD_getMaxBacklight(
   '     PhidgetLCDHandle ch,
   '     double *maxBacklight
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     minbacklight: The backlight value
   ' Return: The Phidget return code
   ' Constant 1.0
   '
   ' The maximum value that Backlight can be set to.
   '
   Public Declare Function PhLcdGetMaxBacklight& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getMaxBacklight" ( _
      ByVal handle&, _
      ByRef maxBacklight#)

   ' ========================================================================
   ' PhidgetLCD_setCharacterBitmap(
   '     PhidgetLCDHandle ch,
   '     PhidgetLCD_Font font,
   '     const char *character,
   '     const uint8_t *bitmap
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     font: The font the character belongs to
   '     character: The character to be changed, in a null-terminated string.
   '     bitmap:  Bitmap array
   ' Return: The Phidget return code
   '
   ' Create a bitmap and select a character to represent it. Now, when you
   ' use the specific character, the bitmap will show in it's place.
   ' Custom characters for fonts User1 and User2 are stored on frame buffers
   ' 1 and 2 respectively. These characters occupy the same space as drawings
   ' on their frame buffer, and will be displayed on-screen if their frame
   ' buffer is flushed. They can also be overwritten by using drawing
   ' functions on their frame buffer, so it is recommended to only use a
   ' given frame buffer for either drawing or storing fonts, but not both.
   '
   ' We recommend using FONT_User2 (frame buffer 2) to store custom fonts,
   ' as it can be saved for later use.
   '
   ' In order to use the custom fonts, you must first define their size
   ' with setFontSize.  A font may use as many characters as will fit on
   ' the screen for the given font size, indicated by getMaxCharacters.
   '
   ' For custom fonts, the character indexing starts with character &h01,
   ' and ascends from there.
   '
   ' Each byte in the bitmap array represents one pixel in row-major order.
   '
   Public Declare Function PhLcdSetCharacterBitmap& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_setCharacterBitmap" ( _
      ByVal handle&, _
      ByVal font&, _
      ByVal strPtrCharacter&, _
      ByRef bitmap() As Byte)

   ' ========================================================================
   ' PhidgetLCD_setCharacterBitmap_async(
   '     PhidgetLCDHandle ch,
   '     PhidgetLCD_Font font,
   '     const char *character,
   '     const uint8_t *bitmap
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '         void *ctx,
   '         PhidgetReturnCode res),
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     font: The font the character belongs to
   '     character: The character to be changed, in a null-terminated string.
   '     bitmap:  Bitmap array
   '     asyncHandler: Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' Create a bitmap and select a character to represent it. Now, when you
   ' use the specific character, the bitmap will show in it's place.
   ' Custom characters for fonts User1 and User2 are stored on frame buffers
   ' 1 and 2 respectively. These characters occupy the same space as drawings
   ' on their frame buffer, and will be displayed on-screen if their frame
   ' buffer is flushed. They can also be overwritten by using drawing
   ' functions on their frame buffer, so it is recommended to only use a
   ' given frame buffer for either drawing or storing fonts, but not both.
   '
   ' We recommend using FONT_User2 (frame buffer 2) to store custom fonts,
   ' as it can be saved for later use.
   '
   ' In order to use the custom fonts, you must first define their size
   ' with setFontSize.  A font may use as many characters as will fit on
   ' the screen for the given font size, indicated by getMaxCharacters.
   '
   ' For custom fonts, the character indexing starts with character &h01,
   ' and ascends from there.
   '
   ' Each byte in the bitmap array represents one pixel in row-major order.
   '
   ' When calling; set asyncHandlerPtr& to
   ' AddressOf PhLcdOnSetCharacterBitmapAsync
   '
   Public Declare Function PhLcdSetCharacterBitmapAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_setCharacterBitmap_async" ( _
      ByVal handle&, _
      ByVal font&, _
      ByVal strPtrCharacter&, _
      ByRef bitmap() As Byte, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)

   ' ========================================================================
   ' PhidgetLCD_getMaxCharacters(
   '     PhidgetLCDHandle ch,
   '     PhidgetLCD_Font font,
   '     int *maxCharacters
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     font: The specified font
   '     maxCharacters: The maximum number of characters for the font
   ' Return: The Phidget return code
   '
   ' The maximum number of characters that can fit on the frame buffer for
   ' the specified font.
   '
   Public Declare Function PhLcdGetMaxCharacters& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getMaxCharacters" ( _
      ByVal handle&, _
      ByVal font&, _
      ByRef maxCharacters&)

   ' ========================================================================
   ' PhidgetLCD_clear(
   '     PhidgetLCDHandle ch
   ' )
   ' Parameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   '
   ' Clears all pixels in the current frame buffer.
   ' Changes made to the frame buffer must be flushed to the LCD screen
   ' using flush .
   '
   Public Declare Function PhLcdClear& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_clear" ( _
      ByVal handle&)

   ' ========================================================================
   ' PhidgetLCD_clear_async(
   '     PhidgetLCDHandle ch
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '         void *ctx,
   '         PhidgetReturnCode res),
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     asyncHandler: Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' Clears all pixels in the current frame buffer.
   ' Changes made to the frame buffer must be flushed to the LCD screen
   ' using flush .
   '
   ' When calling; set asyncHandlerPtr& to AddressOf PhLcdOnClearAsync
   '
   Public Declare Function PhLcdClearAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_clear_async" ( _
      ByVal handle&, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)

   ' ========================================================================
   ' PhidgetLCD_getContrast(
   '     PhidgetLCDHandle ch,
   '     double *contrast
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     contrast: The contrast value
   ' Return: The Phidget return code
   ' Default: 0.25
   '
   ' Contrast level of the text or graphic pixels.
   ' A higher contrast will make the image darker.
   ' Contrast is bounded by MinContrast and MaxContrast.
   '
   Public Declare Function PhLcdGetContrast& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getContrast" ( _
      ByVal handle&, _
      ByRef contrast#)

   ' ========================================================================
   ' PhidgetLCD_setContrast(
   '     PhidgetLCDHandle ch,
   '     double contrast
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     contrast: The contrast value
   ' Return: The Phidget return code
   ' Default: 0.25
   '
   ' Contrast level of the text or graphic pixels.
   ' A higher contrast will make the image darker.
   ' Contrast is bounded by MinContrast and MaxContrast.
   '
   Public Declare Function PhLcdSetContrast& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_setContrast" ( _
      ByVal handle&, _
      ByVal contrast#)

   ' ========================================================================
   ' PhidgetLCD_getMinContrast(
   '     PhidgetLCDHandle ch,
   '     double *minContrast
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     mimContrast: The contrast value
   ' Return: The Phidget return code
   ' Default: 0.0
   '
   ' The minimum value that Contrast can be set to.
   '
   Public Declare Function PhLcdGetMinContrast& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getMinContrast" ( _
      ByVal handle&, _
      ByRef minContrast#)

   ' ========================================================================
   ' PhidgetLCD_getMaxContrast(
   '     PhidgetLCDHandle ch,
   '     double *maxContrast
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     maxContrast: The contrast value
   ' Return: The Phidget return code
   ' Constant: 1.0
   '
   ' The maximum value that Contrast can be set to.
   '
   Public Declare Function PhLcdGetMaxContrast& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getMaxContrast" ( _
      ByVal handle&, _
      ByRef maxContrast#)

   ' ========================================================================
   ' PhidgetLCD_copy(
   '     PhidgetLCDHandle ch,
   '     int sourceFramebuffer,
   '     int destFramebuffer,
   '     int sourceX1,
   '     int sourceY1,
   '     int sourceX2,
   '     int sourceY2,
   '     int destX,
   '     int destY,
   '     int inverted
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     sourceFramebuffer: Index number of the frame buffer containing
   '                        the source rectangle
   '     destFramebuffer: Index number of the frame buffer containing
   '                      the destination rectangle
   '     sourceX1: X coordinate of upper left corner of source rectangle
   '     sourceY1: Y coordinate of upper left corner of source rectangle
   '     sourceX2: X coordinate of bottom right corner of source rectangle
   '     sourceY2: Y coordinate of bottom right corner of source rectangle
   '     destX: X coordinate of upper left corner of destination rectangle
   '     destY: Y coordinate of upper left corner of destination rectangle
   '     inverted: If true, copied pixels are inverted
   ' Return: The Phidget return code
   '
   ' Copies all pixels from a specified rectangular region to another.
   '
   Public Declare Function PhLcdCopy& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_copy" ( _
      ByVal handle&, _
      ByVal sourceFramebuffer&, _
      ByVal destFramebuffer&, _
      ByVal sourceX1&, _
      ByVal sourceY1&, _
      ByVal sourceX2&, _
      ByVal sourceY2&, _
      ByVal destX&, _
      ByVal destY&, _
      ByVal inverted&)

   ' ========================================================================
   ' PhidgetLCD_copy_async(
   '     PhidgetLCDHandle ch,
   '     int sourceFramebuffer,
   '     int destFramebuffer,
   '     int sourceX1,
   '     int sourceY1,
   '     int sourceX2,
   '     int sourceY2,
   '     int destX,
   '     int destY,
   '     int inverted
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '        void *ctx,
   '        PhidgetReturnCode res),
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     sourceFramebuffer: Index number of the frame buffer containing
   '                        the source rectangle
   '     destFramebuffer: Index number of the frame buffer containing
   '                      the destination rectangle
   '     sourceX1: X coordinate of upper left corner of source rectangle
   '     sourceY1: Y coordinate of upper left corner of source rectangle
   '     sourceX2: X coordinate of bottom right corner of source rectangle
   '     sourceY2: Y coordinate of bottom right corner of source rectangle
   '     destX: X coordinate of upper left corner of destination rectangle
   '     destY: Y coordinate of upper left corner of destination rectangle
   '     inverted: If true, copied pixels are inverted
   '     asyncHandler: Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' Copies all pixels from a specified rectangular region to another.
   '
   ' When calling; set asyncHandlerPtr& to AddressOf PhLcdOnCopyAsync
   '
   Public Declare Function PhLcdCopyAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_copy_async" ( _
      ByVal handle&, _
      ByVal sourceFramebuffer&, _
      ByVal destFramebuffer&, _
      ByVal sourceX1&, _
      ByVal sourceY1&, _
      ByVal sourceX2&, _
      ByVal sourceY2&, _
      ByVal destX&, _
      ByVal destY&, _
      ByVal inverted&, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)

   ' ========================================================================
   '  PhidgetLCD_drawLine(
   '     PhidgetLCDHandle ch,
   '     int  x1,
   '     int  y1,
   '     int  x2,
   '     int  y2
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '        void *ctx,
   '        PhidgetReturnCode res),
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     x1: X coordinate of the first point
   '     y1: Y coordinate of the first point
   '     x2: X coordinate of the second point
   '     y2: Y coordinate of the second point
   ' Return: The Phidget return code
   '
   ' Draws a straight line in the current frame buffer between two
   ' specified points
   ' Changes made to the frame buffer must be flushed to the LCD
   ' screen using flush.
   '
   Public Declare Function PhLcdDrawLine& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_drawLine" ( _
      ByVal handle&, _
      ByVal x1&, _
      ByVal y1&, _
      ByVal x2&, _
      ByVal y2&)

   ' ========================================================================
   '  PhidgetLCD_drawLine_async(
   '     PhidgetLCDHandle ch,
   '     int  x1,
   '     int  y1,
   '     int  x2,
   '     int  y2
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '        void *ctx,
   '        PhidgetReturnCode res),
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     x1: X coordinate of the first point
   '     y1: Y coordinate of the first point
   '     x2: X coordinate of the second point
   '     y2: Y coordinate of the second point
   '     asyncHandler: Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' Draws a straight line in the current frame buffer between two
   ' specified points
   ' Changes made to the frame buffer must be flushed to the LCD
   ' screen using flush.
   '
   ' When calling; set asyncHandlerPtr& to AddressOf PhLcdOnDrawLineAsync
   '
   Public Declare Function PhLcdDrawLineAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_drawLine_async" ( _
      ByVal handle&, _
      ByVal x1&, _
      ByVal y1&, _
      ByVal x2&, _
      ByVal y2&, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)

   ' ========================================================================
   '  PhidgetLCD_drawPixel(
   '     PhidgetLCDHandle ch,
   '     int  x,
   '     int  y,
   '     PhidgetLCD_PixelState pixelState
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     x: X coordinate of the first point
   '     y: Y coordinate of the first point
   '     pixelState: The new state of the pixel.   0=Off  1=On  2=Invert
   ' Return: The Phidget return code
   '
   ' Draws, erases, or inverts a single specified pixel.
   ' Changes made to the frame buffer must be flushed to the LCD
   ' screen using flush.
   '
   Public Declare Function PhLcdDrawPixel& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_drawPixel" ( _
      ByVal handle&, _
      ByVal x&, _
      ByVal y&, _
      ByVal pixelState&)

   ' ========================================================================
   '  PhidgetLCD_drawPixel_async(
   '     PhidgetLCDHandle ch,
   '     int  x,
   '     int  y,
   '     PhidgetLCD_PixelState pixelState
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '        void *ctx,
   '        PhidgetReturnCode res),
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     x: X coordinate of the first point
   '     y: Y coordinate of the first point
   '     pixelState: The new state of the pixel.   0=Off  1=On  2=Invert
   '     asyncHandler: Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' Draws, erases, or inverts a single specified pixel.
   ' Changes made to the frame buffer must be flushed to the LCD
   ' screen using flush.
   '
   ' When calling; set asyncHandlerPtr& to AddressOf PhLcdOnDrawPixelAsync
   '
   Public Declare Function PhLcdDrawPixelAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_drawPixel_async" ( _
      ByVal handle&, _
      ByVal x&, _
      ByVal y&, _
      ByVal pixelState&, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)

   ' ========================================================================
   '  PhidgetLCD_drawRect(
   '     PhidgetLCDHandle ch,
   '     int  x1,
   '     int  y1,
   '     int  x2,
   '     int  y2,
   '     int  filled,
   '     int  inverted
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     x1: The X coordinate of the top-left corner of the rectangle
   '     y1: The Y coordinate of the top-left corner of the rectangle
   '     x2: The X coordinate of the bottom-right corner of the rectangle
   '     y2: The Y coordinate of the bottom-right corner of the rectangle
   '     filled: If true, the rectangle will be solid. If false, just a
   '             single pixel outline.
   '     inverted: If true, clears the region instead of drawing
   ' Return: The Phidget return code
   '
   ' Draws a rectangle in the current frame buffer using the specified points
   ' Changes made to the frame buffer must be flushed to the LCD
   ' screen using flush.
   '
   Public Declare Function PhLcdDrawRect& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_drawRect" ( _
      ByVal handle&, _
      ByVal x1&, _
      ByVal y1&, _
      ByVal x2&, _
      ByVal y2&, _
      ByVal filled&, _
      ByVal inverted&)

   ' ========================================================================
   '  PhidgetLCD_drawPixel_async(
   '     PhidgetLCDHandle ch,
   '     int  x1,
   '     int  y1,
   '     int  x2,
   '     int  y2,
   '     int  filled,
   '     int  inverted
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '        void *ctx,
   '        PhidgetReturnCode res),
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     x1: The X coordinate of the top-left corner of the rectangle
   '     y1: The Y coordinate of the top-left corner of the rectangle
   '     x2: The X coordinate of the bottom-right corner of the rectangle
   '     y2: The Y coordinate of the bottom-right corner of the rectangle
   '     filled: If true, the rectangle will be solid. If false, just a
   '             single pixel outline.
   '     inverted: If true, clears the region instead of drawing
   '     asyncHandler: Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' Draws, erases, or inverts a single specified pixel.
   ' Changes made to the frame buffer must be flushed to the LCD
   ' screen using flush.
   '
   ' When calling; set asyncHandlerPtr& to AddressOf PhLcdOnDrawRectAsync
   '
   Public Declare Function PhLcdDrawRectAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_drawRect_async" ( _
      ByVal handle&, _
      ByVal x&, _
      ByVal y&, _
      ByVal pixelState&, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)

   ' ========================================================================
   ' PhidgetLCD_flush(
   '     PhidgetLCDHandle ch,
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     asyncHandler: Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' Flushes the current frame buffer to the LCD screen.
   ' Flush cannot be performed in sleep mode.
   '
   Public Declare Function PhLcdFlush& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_flush" ( _
      ByVal handle&)

   ' ========================================================================
   ' PhidgetLCD_flush_async(
   '     PhidgetLCDHandle ch,
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '        void *ctx,
   '        PhidgetReturnCode res),
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   '
   ' Flushes the current frame buffer to the LCD screen.
   ' Flush cannot be performed in sleep mode.
   '
   ' When calling; set asyncHandlerPtr& to AddressOf PhLcdOnFlushAsync
   '
   Public Declare Function PhLcdFlushAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_flush_async" ( _
      ByVal handle&, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)

   ' ========================================================================
   ' PhidgetLCD_getFontSize(
   '     PhidgetLCDHandle ch,
   '     PhidgetLCD_Font font,
   '     int *width
   '     int *height
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     font: The specified font
   '     width: The width of the font
   '     height: The height of the font
   ' Return: The Phidget return code
   '
   ' Gets the size of the specified font.
   '
   Public Declare Function PhLcdGetFontSize& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getFontSize" ( _
      ByVal handle&, _
      ByVal font&, _
      ByRef width&, _
      ByRef height&)

   ' ========================================================================
   ' PhidgetLCD_setFontSize(
   '     PhidgetLCDHandle ch,
   '     PhidgetLCD_Font font,
   '     int width
   '     int height
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     font: The specified font
   '     width: The width of the font
   '     height: The height of the font
   ' Return: The Phidget return code
   '
   ' Sets the size of the specified font.
   '
   Public Declare Function PhLcdSetFontSize& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_setFontSize" ( _
      ByVal handle&, _
      ByVal font&, _
      ByVal width&, _
      ByVal height&)

   ' ========================================================================
   ' PhidgetLCD_getFrameBuffer(
   '     PhidgetLCDHandle ch,
   '     int *frameBuffer
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     frameBuffer: The current frame buffer
   ' Return: The Phidget return code
   ' Default: 0
   '
   ' The frame buffer that is currently being used.
   ' Commands sent to the device are performed on this buffer.
   '
   Public Declare Function PhLcdGetFrameBuffer& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getFrameBuffer" ( _
      ByVal handle&, _
      ByRef frameBuffer&)

   ' ========================================================================
   ' PhidgetLCD_setFrameBuffer(
   '     PhidgetLCDHandle ch,
   '     int frameBuffer
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     frameBuffer: The current frame buffer
   ' Return: The Phidget return code
   ' Default: 0
   '
   ' The frame buffer that is currently being used.
   ' Commands sent to the device are performed on this buffer.
   '
   Public Declare Function PhLcdSetFrameBuffer& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_setFrameBuffer" ( _
      ByVal handle&, _
      ByVal frameBuffer&)

   ' ========================================================================
   ' PhidgetLCD_setFrameBuffer_async(
   '     PhidgetLCDHandle ch,
   '     int frameBuffer
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '        void *ctx,
   '        PhidgetReturnCode res),
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     frameBuffer: The current frame buffer
   '     asyncHandler: Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' The frame buffer that is currently being used.
   ' Commands sent to the device are performed on this buffer.
   '
   ' When calling; set asyncHandlerPtr& to AddressOf PhLcdOnSetFrameBufferAsync
   '
   Public Declare Function PhLcdSetFrameBufferAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_setFrameBuffer_async" ( _
      ByVal handle&, _
      ByVal frameBuffer&, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)

   ' ========================================================================
   ' PhidgetLCD_getHeight(
   '     PhidgetLCDHandle ch,
   '     int *height
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     frameBuffer: The current frame buffer
   ' Return: The Phidget return code
   ' Constant: 64
   '
   ' The height of the LCD screen attached to the channel.
   '
   Public Declare Function PhLcdGetHeight& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getHeight" ( _
      ByVal handle&, _
      ByRef height&)

   ' ========================================================================
   ' PhidgetLCD_saveFrameBuffer(
   '     PhidgetLCDHandle ch,
   '     int frameBuffer
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     frameBuffer: The current frame buffer
   ' Return: The Phidget return code
   '
   ' Writes the specified frame buffer to flash memory
   ' Use sparingly.  The flash memory is only designed to be written
   ' to 10,000 times before it may become unusable.  This method can
   ' only be called one time each time the channel is opened.
   ' Frame buffer 2 is the only frame buffer that can be saved
   ' on the LCD1100.
   '
   Public Declare Function PhLcdSaveFrameBuffer& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_saveFrameBuffer" ( _
      ByVal handle&, _
      ByVal frameBuffer&)

   ' ========================================================================
   ' PhidgetLCD_saveFrameBuffer_async(
   '     PhidgetLCDHandle ch,
   '     int frameBuffer
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '        void *ctx,
   '        PhidgetReturnCode res),
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     frameBuffer: The current frame buffer
   '     asyncHandler: Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' Writes the specified frame buffer to flash memory
   ' Use sparingly.  The flash memory is only designed to be written
   ' to 10,000 times before it may become unusable.  This method can
   ' only be called one time each time the channel is opened.
   ' Frame buffer 2 is the only frame buffer that can be saved
   ' on the LCD1100.
   '
   ' When calling; set asyncHandlerPtr& to AddressOf PhLcdOnSaveFrameBufferAsync
   '
   Public Declare Function PhLcdSaveFrameBufferAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_saveFrameBuffer_async" ( _
      ByVal handle&, _
      ByVal frameBuffer&, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)

   ' ========================================================================
   ' PhidgetLCD_getScreenSize(
   '     PhidgetLCDHandle ch,
   '     int *screenSize
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     screenSize: The screen size
   ' Return: The Phidget return code
   ' Constant: SCREEN_SIZE_64x128
   '
   ' The height of the LCD screen attached to the channel.
   '
   Public Declare Function PhLcdGetScreenSize& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getScreenSize" ( _
      ByVal handle&, _
      ByRef screenSize&)

   ' ========================================================================
   ' PhidgetLCD_getSleeping(
   '     PhidgetLCDHandle ch,
   '     int *sleeping
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     sleeping: The sleep status
   ' Return: The Phidget return code
   ' Default: 1
   '
   ' The on/off state of Sleeping . Putting the device to sleep turns
   ' off the display and backlight in order to save power.
   ' The device will still take commands while asleep, and will wake up
   ' if the screen is flushed, or if the contrast or backlight are changed.
   ' When the device wakes up, it will return to its last known state,
   ' taking into account any changes that happened while asleep.
   '
   Public Declare Function PhLcdGetSleeping& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getSleeping" ( _
      ByVal handle&, _
      ByRef sleeping&)

   ' ========================================================================
   ' PhidgetLCD_setSleeping(
   '     PhidgetLCDHandle ch,
   '     int sleeping
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     sleeping: The sleep status
   ' Return: The Phidget return code
   ' Default: 1
   '
   ' The on/off state of Sleeping . Putting the device to sleep turns
   ' off the display and backlight in order to save power.
   ' The device will still take commands while asleep, and will wake up
   ' if the screen is flushed, or if the contrast or backlight are changed.
   ' When the device wakes up, it will return to its last known state,
   ' taking into account any changes that happened while asleep.
   '
   Public Declare Function PhLcdSetSleeping& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_setSleeping" ( _
      ByVal handle&, _
      ByVal sleeping&)

   ' ========================================================================
   ' PhidgetLCD_getWidth(
   '     PhidgetLCDHandle ch,
   '     int *width
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     width: The width value
   ' Return: The Phidget return code
   ' Constant: 128
   '
   ' The width of the LCD screen attached to the channel.
   '
   Public Declare Function PhLcdGetWidth& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_getWidth" ( _
      ByVal handle&, _
      ByRef width&)

   ' ========================================================================
   ' PhidgetLCD_writeBitmap(
   '     PhidgetLCDHandle ch,
   '     int  xPosition,
   '     int  yPosition,
   '     int  xSize,
   '     int  ySize,
   '     const uint8_t *bitmap
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     xPosition: The X coordinate of the bitmap
   '     yPosition: The Y coordinate of the bitmap
   '     xSize: The length of each row in the bitmap
   '     ySize: The number of rows in the bitmap
   '     bitmap: The bitmap to be drawn
   ' Return: The Phidget return code
   '
   ' Draws a bitmap to the current frame buffer at the given location.
   ' Each byte in the array represents one pixel in row-major order.
   ' Changes made to the frame buffer must be flushed to the LCD screen
   ' using flush.
   '
   ' When calling; set bitmap& to VarPtr(imageBuf(0, 0)
   '               imageBuf is byte array of 128 by 64
   
   Public Declare Function PhLcdWriteBitmap& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_writeBitmap" ( _
      ByVal handle&, _
      ByVal xPosition&, _
      ByVal yPosition&, _
      ByVal xSize&, _
      ByVal ySize&, _
      ByVal bitmap&)

   ' ========================================================================
   ' PhidgetLCD_writeBitmap_async(
   '     PhidgetLCDHandle ch,
   '     int  xPosition,
   '     int  yPosition,
   '     int  xSize,
   '     int  ySize,
   '     const uint8_t *bitmap
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '        void *ctx,
   '        PhidgetReturnCode res),
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     xPosition: The X coordinate of the bitmap
   '     yPosition: The Y coordinate of the bitmap
   '     xSize: The length of each row in the bitmap
   '     ySize: The number of rows in the bitmap
   '     bitmap: The bitmap to be drawn
   '     asyncHandler: Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' Draws a bitmap to the current frame buffer at the given location.
   ' Each byte in the array represents one pixel in row-major order.
   ' Changes made to the frame buffer must be flushed to the LCD screen
   ' using flush.
   '
   ' When calling; set bitmap& to VarPtr(imageBuf(0, 0)
   '               imageBuf is byte array of 128 by 64
   '               set asyncHandlerPtr& to AddressOf PhLcdOnWriteBitmapAsync
   '               set userData& to indx&
   
   Public Declare Function PhLcdWriteBitmapAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_writeBitmap_async" ( _
      ByVal handle&, _
      ByVal xPosition&, _
      ByVal yPosition&, _
      ByVal xSize&, _
      ByVal ySize&, _
      ByVal bitmap&, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)

   ' ========================================================================
   ' PhidgetLCD_writeText(
   '     PhidgetLCDHandle ch,
   '     PhidgetLCD_Font font,
   '     int xPosition,
   '     int yPosition,
   '     const char *text
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     font: The font of the text
   '     xPosition: The X position of the start of the text string
   '     yPosition: The Y position of the start of the text string
   '     text: The text to be written
   ' Return: The Phidget return code
   '
   ' Writes text to the current frame buffer at the specified location
   ' Changes made to the frame buffer must be flushed to the LCD screen
   ' using flush.
   '
   Public Declare Function PhLcdWriteText& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_writeText" ( _
      ByVal handle&, _
      ByVal font&, _
      ByVal xPosition&, _
      ByVal yPosition&, _
      ByVal strPtrText&)

   ' ========================================================================
   ' PhidgetLCD_writeText_async(
   '     PhidgetLCDHandle ch,
   '     PhidgetLCD_Font font,
   '     int xPosition,
   '     int yPosition,
   '     const char *text
   '     void (CCONV *asyncHandler)
   '        (PhidgetHandle phid,
   '        void *ctx,
   '        PhidgetReturnCode res),
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     font: The font of the text
   '     xPosition: The X position of the start of the text string
   '     yPosition: The Y position of the start of the text string
   '     text: The text to be written
   '     asyncHandler: Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' Writes text to the current frame buffer at the specified location
   ' Changes made to the frame buffer must be flushed to the LCD screen
   ' using flush.
   '
   ' When calling; set asyncHandlerPtr& to AddressOf PhLcdOnWriteTextAsync
   '
   Public Declare Function PhLcdWriteTextAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetLCD_writeText_async" ( _
      ByVal handle&, _
      ByVal font&, _
      ByVal xPosition&, _
      ByVal yPosition&, _
      ByVal strPtrText&, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)



' ===========================================================================
' Event Callbacks, Subs, and Functions
'    When using any of the "Async" functions, the DLL will complete the
'    process and create an EVENT for one of the following functions.
' ===========================================================================

' ===========================================================================
Public Static Function PhLcdOnSetCharacterBitmapAsync&( _
   ByVal handle&, _
   ByVal userData&)

   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhLcdOnSetCharacterBitmapAsync = 0 ' Return Code

End Function

' ===========================================================================
Public Static Function PhLcdOnClearAsync&( _
   ByVal handle&, _
   ByVal userData&)

   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhLcdOnClearAsync = 0 ' Return Code

End Function

' ===========================================================================
Public Static Function PhLcdOnCopyAsync&( _
   ByVal handle&, _
   ByVal userData&)

   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhLcdOnCopyAsync = 0 ' Return Code

End Function

' ===========================================================================
Public Static Function PhLcdOnDrawLineAsync&( _
   ByVal handle&, _
   ByVal userData&)

   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhLcdOnDrawLineAsync = 0 ' Return Code

End Function

' ===========================================================================
Public Static Function PhLcdOnDrawPixelAsync&( _
   ByVal handle&, _
   ByVal userData&)

   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhLcdOnDrawPixelAsync& = 0 ' Return Code

End Function

' ===========================================================================
Public Static Function PhLcdOnDrawRectAsync&( _
   ByVal handle&, _
   ByVal userData&)

   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhLcdOnDrawRectAsync& = 0 ' Return Code

End Function

' ===========================================================================
Public Static Function PhLcdOnFlushAsync&( _
   ByVal handle&, _
   ByVal userData&)

   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhLcdOnFlushAsync& = 0 ' Return Code

End Function

' ===========================================================================
Public Static Function PhLcdOnSetFrameBufferAsync( _
   ByVal handle&, _
   ByVal userData&)

   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhLcdOnSetFrameBufferAsync = 0 ' Return Code

End Function

' ===========================================================================
Public Static Function PhLcdOnSaveFrameBufferAsync&( _
   ByVal handle&, _
   ByVal userData&)

   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhLcdOnSaveFrameBufferAsync& = 0 ' Return Code

End Function

' ===========================================================================
Public Static Function PhLcdOnWriteBitmapAsync&( _
   ByVal handle&, _
   ByVal userData&)
   
   'Dim stat&
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   'stat& = PhLcdFlushAsync&(Dev(userData&).hndl, _
           AddressOf PhLcdOnFlushAsync, userData&)
   ' ===================================

   PhLcdOnWriteBitmapAsync& = 0 ' Return Code

End Function

' ===========================================================================
Public Static Function PhLcdOnWriteTextAsync&( _
   ByVal handle&, _
   ByVal userData&)

   'Dim stat&
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   'stat& = PhLcdFlushAsync&(Dev(userData&).hndl, _
           AddressOf PhLcdOnFlushAsync, userData&)
   ' ===================================

   PhLcdOnWriteTextAsync& = 0 ' Return Code
   
End Function

' ==========================================================================
' PHIDGET LCD STARTUP
' ==========================================================================
Public Static Sub InitPhLcd(indx&, stat&)
   ' Enter:    ioData        (Dev(indx&))
   ' Return:   Dev(indx&).hndl&  (Dev(indx&).hndl&)
   '           stat&  0=OK
   
   ' Create, configure, and open a LCD channel.
   ' stat& = 0 if the program exits successfully
   
   Dim attached&
   
   On Error Resume Next
   stat& = 0& ' Set default OK
   
   ' ===================================
   ' Create an instance of a Phidget channel.  Once created the
   ' channel may need to be configured to match the physical
   ' device and channel you want to attach to, and then be
   ' opened by calling Phidget_open().  The channel handle must
   ' be released with PhidgetDigitalOutput_delete().
   '
   ' stat& (Error Code) will always be 0 if no Errors
   '
   stat& = PhLcdCreate&(Dev(indx&).hndl) ' Create new handle
   If stat& <> 0 Then GoTo HandleErr
   
   ' SET SERIAL NUMBER
   stat& = PhSetDeviceSerialNumber(Dev(indx&).hndl, Dev(indx&).hubSn)
   If stat& <> 0 Then GoTo HandleErr

   ' SET ISHUB (0 if hub port is connected to a Phidget module)
   stat& = PhSetIsHubPortDevice(Dev(indx&).hndl, Dev(indx&).isHub)
   If stat& <> 0 Then GoTo HandleErr

   ' SET HUB PORT
   stat& = PhSetHubPort(Dev(indx&).hndl, Dev(indx&).port)
   If stat& <> 0 Then GoTo HandleErr

   ' SET CHANNEL
   stat& = PhSetChannel(Dev(indx&).hndl, Dev(indx&).ch)
   If stat& <> 0 Then GoTo HandleErr
   
   ' Setup the handlers to be run when the device is plugged in,
   ' opened from software, unplugged, closed from software, or
   ' generates an error.
   ' SETUP ATTACH EVENT
   stat& = PhSetOnAttach&(Dev(indx&).hndl, _
               AddressOf PhOnAttach, _
               indx&) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' SETUP DETACH EVENT
   stat& = PhSetOnDetach&(Dev(indx&).hndl, _
               AddressOf PhOnDetach, _
               indx&) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' SETUP ERROR EVENT
   ' Using common Error Handler for all phidget modules.
   stat& = PhSetOnError&(Dev(indx&).hndl, _
               AddressOf PhOnError, _
               indx&) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' OPEN CHANNEL
   'stat& = PhOpen(Dev(indx&).hndl)
   
   ' OPEN CHANNEL & WAIT FOR ATTACH with timeout
   stat& = PhOpenWaitForAttachment(Dev(indx&).hndl, timeOutMs&)
   If stat& <> 0 Then GoTo HandleErr
   
   ' Setup initial pramaters
   stat& = PhGetAttached&(Dev(indx&).hndl, attached&)
   If attached& = 1 Then
      stat& = PhLcdSetBacklight(Dev(indx&).hndl, 0.5)
      If stat& = 0 Then Form1.LcdBackLightHScroll.Value = 50
      
      stat& = PhLcdSetContrast&(Dev(indx&).hndl, 0.3)
      If stat& = 0 Then Form1.LcdContrastHScroll.Value = 30
      
      stat& = PhLcdSetSleeping&(Dev(indx&).hndl, 0&)
   End If
   
   GoTo ExitSub
HandleErr:
   Call Form1.DisplayStatus(indx&, stat&, "InitLcd")

ExitSub:

End Sub

















