Attribute VB_Name = "PhVintStep"
Option Explicit
   
   ' =======================================================================
   ' 2.5A Stepper Phidget
   ' =======================================================================
   ' C STC1001_0 - 2.5A Stepper Phidget 100 Stepper API
        
   ' The Stepper class powers and controls the stepper motor connected
   ' to the Phidget controller, allowing you to change the position,
   ' velocity, acceleration, and current limit.
   '
   ' NOTE:  Will not Attach unless external stepper power is on.
   
   ' =======================================================================
   ' Enumerations:
   ' Method of motor control
   ' =======================================================================
   Global Const CONTROL_MODE_STEP& = 0
   '     Control the motor by setting a target position.
   '
   Global Const CONTROL_MODE_RUN& = 1
   '     Control the motor by selecting a target velocity
   '     (sign indicates direction).  The motor will rotate
   '     continously in the chosen direction.
   
   ' =======================================================================
   ' Functions:
   ' =======================================================================
   
   ' =======================================================================
   ' PhidgetStepper_create(
   '     PhidgetStepperHandle *ch
   ' )
   ' Parameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   '
   ' PhidgetStepper_create() creates an instance of a Phidget channel.
   ' Once created the channel may need to be configured to match the
   ' physical device and channel index you want to attach to, and then
   ' be opened by calling Phidget_open().  The Phidget channel handle
   ' must be released with PhidgetStepper_delete().
   '
   Public Declare Function PhStepCreate& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_create" ( _
      ByRef handle&)
   
   ' =======================================================================
   ' PhidgetStepper_delete(
   '     PhidgetStepperHandle *ch
   ' )
   ' Parameters:
   '     ch: The channel handle
   ' Return: The Phidget return code
   '
   ' PhidgetStepper_delete() releases a Phidget channel handle created
   ' with PhidgetStepper_create().  If the channel is open when
   ' PhidgetStepper_delete() is called, it will be closed.
   '
   Public Declare Function PhStepDelete& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_delete" ( _
      ByRef handle&)
   
   ' =======================================================================
   ' PhidgetStepper_getAcceleration(
   '     PhidgetStepperHandle ch,
   '     double *acceleration
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     acceleration: The acceleration value
   ' Return: The Phidget return code
   ' Default: 10,000.0
   '
   ' The rate at which the controller can change the motor's Velocity.
   ' Units for Position, Velocity, and Acceleration can be set by the
   ' user through the RescaleFactor.  The RescaleFactor allows you to
   ' use more intuitive units such as rotations, or degrees.
   '
   Public Declare Function PhStepGetAcceleration& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getAcceleration" ( _
      ByVal handle&, _
      ByRef acceleration#)
   
   ' =======================================================================
   ' PhidgetStepper_setAcceleration(
   '     PhidgetStepperHandle ch,
   '     double acceleration
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     acceleration: The acceleration value
   ' Return: The Phidget return code
   ' Default: 10,000.0
   '
   ' The rate at which the controller can change the motor's Velocity.
   ' Units for Position, Velocity, and Acceleration can be set by the
   ' user through the RescaleFactor.  The RescaleFactor allows you to
   ' use more intuitive units such as rotations, or degrees.
   '
   Public Declare Function PhStepSetAcceleration& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setAcceleration" ( _
      ByVal handle&, _
      ByVal acceleration#)
   
   ' =======================================================================
   ' PhidgetStepper_getMinAcceleration(
   '     PhidgetStepperHandle ch,
   '     double *minAcceleration
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     minAcceleration: The acceleration value
   ' Return: The Phidget return code
   ' Constant: 2.0
   '
   ' The minimum value that Acceleration can be set to.
   ' Units for Position, Velocity, and Acceleration can be set by the user
   ' through the RescaleFactor.  The RescaleFactor allows you to use more
   ' intuitive units such as rotations, or degrees.
   '
   Public Declare Function PhStepGetMinAcceleration& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getMinAcceleration" ( _
      ByVal handle&, _
      ByRef minAcceleration#)
   
   ' =======================================================================
   ' PhidgetStepper_getMaxAcceleration(
   '     PhidgetStepperHandle ch,
   '     double *maxAcceleration
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     maxAcceleration: The acceleration value
   ' Return: The Phidget return code
   ' Constant: 10,000,000.0
   '
   ' The maximum value that Acceleration can be set to.
   ' Units for Position, Velocity, and Acceleration can be set by the user
   ' through the RescaleFactor.  The RescaleFactor allows you to use more
   ' intuitive units such as rotations, or degrees.
   '
   Public Declare Function PhStepGetMaxAcceleration& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getMaxAcceleration" ( _
      ByVal handle&, _
      ByRef maxAcceleration#)
   
   ' =======================================================================
   ' PhidgetStepper_getControlMode(
   '     PhidgetStepperHandle ch,
   '     PhidgetStepper_ControlMode *controlMode
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     controlMode: The control mode value
   ' Return: The Phidget return code
   ' Default: CONTROL_MODE_STEP
   '
   ' Use step mode when you want to set a TargetPosition for the Stepper
   ' motor.  Use run mode when you simply want the Stepper motor to rotate
   ' continuously in a specific direction.
   '
   Public Declare Function PhStepGetControlMode& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getControlMode" ( _
      ByVal handle&, _
      ByRef controlMode&)
   
   ' =======================================================================
   ' PhidgetStepper_setControlMode(
   '     PhidgetStepperHandle ch,
   '     PhidgetStepper_ControlMode controlMode
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     controlMode: The control mode value
   ' Return: The Phidget return code
   ' Default: CONTROL_MODE_STEP
   '
   ' Use step mode when you want to set a TargetPosition for the Stepper
   ' motor.  Use run mode when you simply want the Stepper motor to rotate
   ' continuously in a specific direction.
   '
   Public Declare Function PhStepSetControlMode& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setControlMode" ( _
      ByVal handle&, _
      ByVal controlMode&)
   
   ' =======================================================================
   ' PhidgetStepper_getCurrentLimit(
   '     PhidgetStepperHandle ch,
   '     double *currentLimit
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     currentLimit: The current limit value
   ' Return: The Phidget return code
   ' Default: 1.0 A
   '
   ' The current through the motor will be limited by the CurrentLimit.
   ' See your Stepper motor's data sheet for more information about what
   ' value the CurrentLimit should be.
   '
   Public Declare Function PhStepGetCurrentLimit& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getCurrentLimit" ( _
      ByVal handle&, _
      ByRef currentLimit#)
   
   ' =======================================================================
   ' PhidgetStepper_setCurrentLimit(
   '     PhidgetStepperHandle ch,
   '     double currentLimit
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     currentLimit: The current limit value
   ' Return: The Phidget return code
   ' Default: 1.0 A
   '
   ' The current through the motor will be limited by the CurrentLimit.
   ' See your Stepper motor's data sheet for more information about what
   ' value the CurrentLimit should be.
   '
   Public Declare Function PhStepSetCurrentLimit& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setCurrentLimit" ( _
      ByVal handle&, _
      ByVal currentLimit#)
   
   ' =======================================================================
   ' PhidgetStepper_getMinCurrentLimit(
   '     PhidgetStepperHandle ch,
   '     double *minCurrentLimit
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     minCurrentLimit: The current limit
   ' Return: The Phidget return code
   ' Constant: 0.0 A
   '
   ' The minimum value that CurrentLimit and HoldingCurrentLimit can be
   ' set to.  Reference your controller's User Guide for more information
   ' about how the HoldingCurrentLimit and CurrentLimit can be used in
   ' your application.
   '
   Public Declare Function PhStepGetMinCurrentLimit& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getMinCurrentLimit" ( _
      ByVal handle&, _
      ByRef minCurrentLimit#)
   
   ' =======================================================================
   ' PhidgetStepper_getMaxCurrentLimit(
   '     PhidgetStepperHandle ch,
   '     double *maxCurrentLimit
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     maxCurrentLimit: The current limit
   ' Return: The Phidget return code
   ' Constant: 2.5 A
   '
   ' The maximum value that CurrentLimit and HoldingCurrentLimit can be
   ' set to.  Reference your controller's User Guide for more information
   ' about how the HoldingCurrentLimit and CurrentLimit can be used in
   ' your application.
   '
   Public Declare Function PhStepGetMaxCurrentLimit& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getMaxCurrentLimit" ( _
      ByVal handle&, _
      ByRef maxCurrentLimit#)
   
   ' =======================================================================
   ' PhidgetStepper_getDataInterval(
   '     PhidgetStepperHandle ch,
   '     uint32_t *dataInterval
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     dataInterval: The data interval value
   ' Return: The Phidget return code
   ' Default: 250 ms
   '
   ' The DataInterval is the time that must elapse before the controller
   ' will fire another PositionChange / VelocityChange event.  The data
   ' interval is bounded by MinDataInterval and MaxDataInterval.
   '
   Public Declare Function PhStepGetDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getDataInterval" ( _
      ByVal handle&, _
      ByRef dataInterval&) ' unsigned integer
   
   ' =======================================================================
   ' PhidgetStepper_setDataInterval(
   '     PhidgetStepperHandle ch,
   '     uint32_t  dataInterval
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     dataInterval: The data interval value
   ' Return: The Phidget return code
   ' Default: 250 ms
   '
   ' The DataInterval is the time that must elapse before the controller
   ' will fire another PositionChange / VelocityChange event.  The data
   ' interval is bounded by MinDataInterval and MaxDataInterval.
   '
   Public Declare Function PhStepSetDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setDataInterval" ( _
      ByVal handle&, _
      ByVal dataInterval&) ' unsigned integer (no vals > &h7FFFFFFF)
   
   ' =======================================================================
   ' PhidgetStepper_getMinDataInterval(
   '     PhidgetStepperHandle ch,
   '     uint32_t *minDataInterval
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     minDataInterval: The data interval value
   ' Return: The Phidget return code
   ' Constant: 100 ms
   '
   ' The minimum value that DataInterval can be set to.
   '
   Public Declare Function PhStepGetMinDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getMinDataInterval" ( _
      ByVal handle&, _
      ByRef minDataInterval&) ' unsigned integer
   
   ' =======================================================================
   ' PhidgetStepper_getMaxDataInterval(
   '     PhidgetStepperHandle ch,
   '     uint32_t *maxDataInterval
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     maxDataInterval: The data interval value
   ' Return: The Phidget return code
   ' Constant: 60,000 ms
   '
   ' The maximum value that DataInterval can be set to.
   '
   Public Declare Function PhStepGetMaxDataInterval& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getMaxDataInterval" ( _
      ByVal handle&, _
      ByRef maxDataInterval&) ' unsigned integer
   
   ' =======================================================================
   ' PhidgetStepper_getEngaged(
   '     PhidgetStepperHandle ch,
   '     int *engaged
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     engaged: The engaged state
   ' Return: The Phidget return code
   ' Default: 0
   '
   ' When this property is true, the controller will supply power to the
   ' motor coils.  The controller must be Engaged in order to move the
   ' Stepper motor, or have it hold position.
   '
   Public Declare Function PhStepGetEngaged& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getEngaged" ( _
      ByVal handle&, _
      ByRef engaged&)
   
   ' =======================================================================
   ' PhidgetStepper_setEngaged(
   '     PhidgetStepperHandle ch,
   '     int  engaged
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     engaged: The engaged state
   ' Return: The Phidget return code
   ' Default: 0
   '
   ' When this property is true, the controller will supply power to the
   ' motor coils.  The controller must be Engaged in order to move the
   ' Stepper motor, or have it hold position.
   '
   Public Declare Function PhStepSetEngaged& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setEngaged" ( _
      ByVal handle&, _
      ByVal engaged&)
   
   ' =======================================================================
   ' PhidgetStepper_getHoldingCurrentLimit(
   '     PhidgetStepperHandle ch,
   '     double *holdingCurrentLimit
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     holdingCurrentLimit: The current value
   ' Return: The Phidget return code
   ' Unit: amperes (A)
   '
   ' The HoldingCurrentLimit will activate when the TargetPosition has
   ' been reached.  It will limit current through the motor.  When the
   ' motor is not stopped, the current through the motor is limited by
   ' the CurrentLimit.  If no HoldingCurrentLimit is specified, the
   ' CurrentLimit value will persist when the motor is stopped.  Reference
   ' your controller's User Guide for more information about how the
   ' HoldingCurrentLimit and CurrentLimit can be used in your application.
   '
   Public Declare Function PhStepGetHoldingCurrentLimit& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getHoldingCurrentLimit" ( _
      ByVal handle&, _
      ByRef holdingCurrentLimit#)
   
   ' =======================================================================
   ' PhidgetStepper_setHoldingCurrentLimit(
   '     PhidgetStepperHandle ch,
   '     double holdingCurrentLimit
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     holdingCurrentLimit: The current value
   ' Return: The Phidget return code
   ' Unit: amperes (A)
   '
   ' The HoldingCurrentLimit will activate when the TargetPosition has
   ' been reached.  It will limit current through the motor.  When the
   ' motor is not stopped, the current through the motor is limited by
   ' the CurrentLimit.  If no HoldingCurrentLimit is specified, the
   ' CurrentLimit value will persist when the motor is stopped.  Reference
   ' your controller's User Guide for more information about how the
   ' HoldingCurrentLimit and CurrentLimit can be used in your application.
   '
   Public Declare Function PhStepSetHoldingCurrentLimit& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setHoldingCurrentLimit" ( _
      ByVal handle&, _
      ByVal holdingCurrentLimit#)
   
   ' =======================================================================
   ' PhidgetStepper_getIsMoving(
   '     PhidgetStepperHandle ch,
   '     int *isMoving
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     isMoving: The moving state
   ' Return: The Phidget return code
   '
   ' IsMoving returns true while the controller is sending commands to
   ' the motor.  Note: there is no feedback to the controller, so it does
   ' not know whether the motor shaft is actually moving or not.
   '
   Public Declare Function PhStepGetIsMoving& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getIsMoving" ( _
      ByVal handle&, _
      ByRef isMoving&)
   
   ' =======================================================================
   ' PhidgetStepper_getPosition(
   '     PhidgetStepperHandle ch,
   '     double *position
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     Position: The position value
   ' Return: The Phidget return code
   '
   ' The most recent position value that the controller has reported.
   ' This value will always be between MinPosition and MaxPosition.
   ' Units for Position, Velocity, and Acceleration can be set by the
   ' user through the RescaleFactor.  The RescaleFactor allows you to
   ' use more intuitive units such as rotations, or degrees.
   '
   Public Declare Function PhStepGetPosition& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getPosition" ( _
      ByVal handle&, _
      ByRef position#)
   
   ' =======================================================================
   ' PhidgetStepper_getMinPosition(
   '     PhidgetStepperHandle ch,
   '     double *minPosition
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     minPosition: The position value
   ' Return: The Phidget return code
   ' Constant: -1,000,000,000,000,000.0
   '
   ' The minimum value that TargetPosition can be set to.  Units for
   ' Position, Velocity, and Acceleration can be set by the user through
   ' the RescaleFactor.  The RescaleFactor allows you to use more
   ' intuitive units such as rotations, or degrees.
   '
   Public Declare Function PhStepGetMinPosition& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getMinPosition" ( _
      ByVal handle&, _
      ByRef minPosition#)
   
   ' =======================================================================
   ' PhidgetStepper_getMaxPosition(
   '     PhidgetStepperHandle ch,
   '     double *maxPosition
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     maxPosition: The position value
   ' Return: The Phidget return code
   ' Constant: 1,000,000,000,000,000.0
   '
   ' The maximum value that TargetPosition can be set to.  Units for
   ' Position, Velocity, and Acceleration can be set by the user through
   ' the RescaleFactor.  The RescaleFactor allows you to use more
   ' intuitive units such as rotations, or degrees.
   '
   Public Declare Function PhStepGetMaxPosition& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getMaxPosition" ( _
      ByVal handle&, _
      ByRef maxPosition#)
   
   ' =======================================================================
   ' PhidgetStepper_addPositionOffset(
   '     PhidgetStepperHandle ch,
   '     double positionOffset
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     positionOffset: Amount to offset the position by
   ' Return: The Phidget return code
   '
   ' Adds an offset (positive or negative) to the current position and
   ' target position.  This is especially useful for zeroing position.
   '
   Public Declare Function PhStepAddPositionOffset& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_addPositionOffset" ( _
      ByVal handle&, _
      ByVal positionOffset#)
   
   ' =======================================================================
   ' PhidgetStepper_getRescaleFactor(
   '     PhidgetStepperHandle ch,
   '     double *rescaleFactor
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     rescaleFactor: The rescale factor value
   ' Return: The Phidget return code
   ' Default: 1.0
   '
   ' Applies a factor to the [user units] per step to all movement
   ' parameters to make the units in your application is more intuitive.
   ' For example, starting from position 0 and setting a new position
   ' with a rescale factor, the stepper will move Position / RescaleFactor
   ' steps.  In this way, units for Position, Velocity, and Acceleration
   ' can be set by the user through the RescaleFactor.
   ' The RescaleFactor allows you to use more intuitive units such as
   ' rotations, or degrees.
   '
   Public Declare Function PhStepGetRescaleFactor& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getRescaleFactor" ( _
      ByVal handle&, _
      ByRef rescaleFactor#)
   
   ' =======================================================================
   ' PhidgetStepper_setRescaleFactor(
   '     PhidgetStepperHandle ch,
   '     double rescaleFactor
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     rescaleFactor: The rescale factor value
   ' Return: The Phidget return code
   ' Default: 1.0
  '
   ' Applies a factor to the [user units] per step to all movement
   ' parameters to make the units in your application is more intuitive.
   ' For example, starting from position 0 and setting a new position
   ' with a rescale factor, the stepper will move Position / RescaleFactor
   ' steps.  In this way, units for Position, Velocity, and Acceleration
   ' can be set by the user through the RescaleFactor.  The RescaleFactor
   ' allows you to use more intuitive units such as rotations, or degrees.
   '
   Public Declare Function PhStepSetRescaleFactor& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setRescaleFactor" ( _
      ByVal handle&, _
      ByVal rescaleFactor#)
   
   ' =======================================================================
   ' PhidgetStepper_getTargetPosition(
   '     PhidgetStepperHandle ch,
   '     double *targetPosition
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     targetPosition: The position value
   ' Return: The Phidget return code
   ' Default: 0.0
   '
   ' If the controller is configured and the TargetPosition is set, the
   ' Stepper motor will move towards the TargetPosition at the specified
   ' Acceleration and Velocity.  TargetPosition is only used when the
   ' ControlMode is set to step mode.  Units for Position, Velocity, and
   ' Acceleration can be set by the user through the RescaleFactor.  The
   ' RescaleFactor allows you to use more intuitive units such as
   ' rotations, or degrees.
   '
   Public Declare Function PhStepGetTargetPosition& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getTargetPosition" ( _
      ByVal handle&, _
      ByRef targetPosition#)
   
   ' =======================================================================
   ' PhidgetStepper_setTargetPosition(
   '     PhidgetStepperHandle ch,
   '     double targetPosition
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     targetPosition: The position value
   ' Return: The Phidget return code
   ' Default: 0.0
   '
   ' If the controller is configured and the TargetPosition is set, the
   ' Stepper motor will move towards the TargetPosition at the specified
   ' Acceleration and Velocity.  TargetPosition is only used when the
   ' ControlMode is set to step mode.  Units for Position, Velocity, and
   ' Acceleration can be set by the user through the RescaleFactor.  The
   ' RescaleFactor allows you to use more intuitive units such as
   ' rotations, or degrees.
   '
   Public Declare Function PhStepSetTargetPosition& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setTargetPosition" ( _
      ByVal handle&, _
      ByVal targetPosition#)
   
   ' =======================================================================
   ' PhidgetStepper_setTargetPosition_async(
   '     PhidgetStepperHandle ch,
   '     double targetPosition,
   '     void (CCONV *asyncHandler)
   '       (PhidgetHandle phid,
   '        void *ctx,
   '        PhidgetReturnCode res),
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     targetPosition: The position value
   '     asyncHandler: Asynchronous completion callback
   '     ctx: User supplied context pointer
   ' Return: The Phidget return code
   '
   ' If the controller is configured and the TargetPosition is set, the
   ' Stepper motor will move towards the TargetPosition at the specified
   ' Acceleration and Velocity.  TargetPosition is only used when the
   ' ControlMode is set to step mode.  Units for Position, Velocity, and
   ' Acceleration can be set by the user through the RescaleFactor.  The
   ' RescaleFactor allows you to use more intuitive units such as
   ' rotations, or degrees.
   '
   ' This function requires PhStepOnSetTargetPosition callback procedure
   ' AsyncHandlerPtr& must be set to AddressOf PhStepOnSetTargetPosition
   '
   Public Declare Function PhStepSetTargetPositionAsync& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setTargetPosition_async" ( _
      ByVal handle&, _
      ByVal targetPosition#, _
      ByVal asyncHandlerPtr&, _
      ByVal userData&)

   ' =======================================================================
   ' PhidgetStepper_getVelocity(
   '     PhidgetStepperHandle ch,
   '     double *velocity
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     velocity: The velocity value
   ' Return: The Phidget return code
   '
   ' The most recent velocity value that the controller has reported.
   ' This value is bounded by MinVelocityLimit and MaxVelocityLimit.
   ' Units for Position, Velocity, and Acceleration can be set by the
   ' user through the RescaleFactor.  The RescaleFactor allows you to
   ' use more intuitive units such as rotations, or degrees.
   '
   Public Declare Function PhStepGetVelocity& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getVelocity" ( _
      ByVal handle&, _
      ByRef velocity#)
   
   ' =======================================================================
   ' PhidgetStepper_getVelocityLimit(
   '     PhidgetStepperHandle ch,
   '     double *velocityLimit
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     velocityLimit: Velocity limit
   ' Return: The Phidget return code
   ' Default: 10,000.0
   '
   ' When moving, the Stepper motor velocity will be limited by this value.
   ' The VelocityLimit is bounded by MinVelocityLimit and MaxVelocityLimit.
   ' When in step mode, the MinVelocityLimit has a value of 0.  This is
   ' because the sign () of the TargetPosition will indicate the direction.
   ' When in run mode, the MinVelocityLimit has a value of -MaxVelocityLimit.
   ' This is because there is no target position, so the direction is
   ' defined by the sign () of the VelocityLimit.  Units for Position,
   ' Velocity, and Acceleration can be set by the user through the
   ' RescaleFactor.  The RescaleFactor allows you to use more intuitive
   ' units such as rotations, or degrees.
   '
   Public Declare Function PhStepGetVelocityLimit& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getVelocityLimit" ( _
      ByVal handle&, _
      ByRef velocityLimit#)
   
   ' =======================================================================
   ' PhidgetStepper_setVelocityLimit(
   '     PhidgetStepperHandle ch,
   '     double velocityLimit
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     velocityLimit: Velocity limit
   ' Return: The Phidget return code
   ' Default: 10,000.0
   '
   ' When moving, the Stepper motor velocity will be limited by this value.
   ' The VelocityLimit is bounded by MinVelocityLimit and MaxVelocityLimit.
   ' When in step mode, the MinVelocityLimit has a value of 0.  This is
   ' because the sign () of the TargetPosition will indicate the direction.
   ' When in run mode, the MinVelocityLimit has a value of -MaxVelocityLimit.
   ' This is because there is no target position, so the direction is defined
   ' by the sign () of the VelocityLimit.  Units for Position, Velocity, and
   ' Acceleration can be set by the user through the RescaleFactor.  The
   ' RescaleFactor allows you to use more intuitive units such as rotations,
   ' or degrees.
   '
   Public Declare Function PhStepSetVelocityLimit& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setVelocityLimit" ( _
      ByVal handle&, _
      ByVal velocityLimit#)
   
   ' =======================================================================
   ' PhidgetStepper_getMinVelocityLimit(
   '     PhidgetStepperHandle ch,
   '     double *minVelocityLimit
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     minVelocityLimit: The velocity limit value
   ' Return: The Phidget return code
   ' Constant: 0.0
   '
   ' The minimum value that VelocityLimit can be set to.
   ' Units for Position, Velocity, and Acceleration can be set by the
   ' user through the RescaleFactor.  The RescaleFactor allows you to
   ' use more intuitive units such as rotations, or degrees.
   '
   Public Declare Function PhStepGetMinVelocityLimit& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getMinVelocityLimit" ( _
      ByVal handle&, _
      ByRef minVelocityLimit#)
   
   ' =======================================================================
   ' PhidgetStepper_getMaxVelocityLimit(
   '     PhidgetStepperHandle ch,
   '     double *maxVelocityLimit
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     maxVelocityLimit: The velocity value
   ' Return: The Phidget return code
   ' Constant: 115,000.0
   '
   ' The maximum value that VelocityLimit can be set to.
   ' Units for Position, Velocity, and Acceleration can be set by the
   ' user through the RescaleFactor.  The RescaleFactor allows you to
   ' use more intuitive units such as rotations, or degrees.
   '
   Public Declare Function PhStepGetMaxVelocityLimit& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_getMaxVelocityLimit" ( _
      ByVal handle&, _
      ByRef maxVelocityLimit#)
   
   ' =======================================================================
   ' Event Setups:
   ' =======================================================================
   
   ' PhidgetStepper_setOnPositionChangeHandler(
   '     PhidgetStepperHandle ch,
   '     PhidgetStepper_OnPositionChangeCallback handler,
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     handler: The callback function
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   '
   ' Assigns a handler that will be called when the PositionChange
   ' event occurs
   '
   ' This function requires PhStepSetOnPositionChange callback procedure
   ' handlerPtr& must be set to AddressOf PhStepSetOnPositionChange
   '
   Public Declare Function PhStepSetOnPositionChangeHandler& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setOnPositionChangeHandler" ( _
      ByVal handle&, _
      ByVal handlerPtr&, _
      ByVal userData&)
   
   ' =======================================================================
   ' PhidgetStepper_setOnStoppedHandler(
   '     PhidgetStepperHandle ch,
   '     PhidgetStepper_OnStoppedCallback handler,
   '     void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     handler: The callback function
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   '
   ' Assigns a handler that will be called when the Stopped event occurs
   '
   ' This function requires PhStepOnStopped callback procedure
   ' handlerPtr& must be set to AddressOf PhStepOnStopped
   '
   Public Declare Function PhStepSetOnStoppedHandler& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setOnStoppedHandler" ( _
      ByVal handle&, _
      ByVal handlerPtr&, _
      ByVal userData&)
   
   ' =======================================================================
   ' PhidgetStepper_setOnVelocityChangeHandler(
   ' PhidgetStepperHandle ch,
   ' PhidgetStepper_OnVelocityChangeCallback handler,
   ' void *ctx
   ' )
   ' Parameters:
   '     ch: The channel handle
   '     handler: The callback function
   '     ctx: User provided context pointer
   ' Return: The Phidget return code
   '
   ' Assigns a handler that will be called when the VelocityChange
   ' event occurs
   '
   ' This function requires PhStepOnVelocityChange callback procedure
   ' handlerPtr& must be set to AddressOf PhStepOnVelocityChange
   '
   Public Declare Function PhStepSetOnVelocityChangeHandler& _
      Lib "Phidget22.dll" _
      Alias "PhidgetStepper_setOnVelocityChangeHandler" ( _
      ByVal handle&, _
      ByVal handlerPtr&, _
      ByVal userData&)
   

' ===========================================================================
' Event Callbacks, Subs, and Functions
' ===========================================================================

' ==========================================================================
' PhidgetStepper_OnPositionChangeCallback
'     void CCONV onPositionChange(
'     PhidgetStepperHandle ch,
'     void *ctx,
'     double position
' )
' Parameters:
'     ch: The channel handle
'     ctx: Pointer to the user context
'     position: The current stepper position
'
' Occurs when the controller updates the stepper motor position.
' This event will still fire even if the motor is blocked from
' physically moving or misses steps.
'
Public Static Function PhStepOnPositionChange&( _
   ByVal handle&, _
   ByVal userData&, _
   ByVal position#)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhStepOnPositionChange& = 0 ' Return Code
   
End Function
   
' ==========================================================================
' PhidgetStepper_OnStoppedCallback
'     void CCONV onStopped(
'     PhidgetStepperHandle ch,
'     void *ctx
' )
' Parameters:
'     ch: The channel handle
'     ctx: Pointer to the user context
'
' Occurs when the motor controller stops.
' The motor may still be physically moving if the inertia is great
' enough to make it misstep.
'
Public Static Function PhStepOnStopped&( _
   ByVal handle&, _
   ByVal userData&)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhStepOnStopped& = 0 ' Return Code
   
End Function
   
' ==========================================================================
' PhidgetStepper_OnVelocityChangeCallback
'     void CCONV onVelocityChange(
'     PhidgetStepperHandle ch,
'     void *ctx,
'     double velocity
' )
' Parameters:
'     ch: The channel handle
'     ctx: Pointer to the user context
'     velocity: Velocity of the stepper.  Sign indicates direction.
'
' Occurs when the stepper motor velocity changes.
'
Public Static Function PhStepOnVelocityChange&( _
   ByVal handle&, _
   ByVal userData&, _
   ByVal velocity#)
   
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================

   PhStepOnVelocityChange& = 0 ' Return Code
   
End Function
   
' ===========================================================================
Public Static Function PhStepOnSetTargetPosition&( _
   ByVal handle&, _
   ByVal userData&)
   '
   ' ========== User code HERE =========
   ' CAUTION: (Using CALLS here might cause .exe to crash)
   ' ===================================
   
   PhStepOnSetTargetPosition& = 0 ' Return Code
   
End Function
   
' ==========================================================================
' PHIDGET ENC1000 ENCODER STARTUP
' ==========================================================================

' ==========================================================================
Public Static Sub InitPhStepper(indx&, stat&)
   ' Startup Stepoder
   
   ' Enter:    Dev(indx&)
   ' Return:   Dev(indx&).hndl&
   '           stat&  0=OK
   
   ' Create, configure, and open an Stepper channel.
   ' stat& = 0 if the program exits successfully
   
   ' ===================================
   ' Create an instance of a Phidget channel.  Once created the
   ' channel may need to be configured to match the physical
   ' device and channel you want to attach to, and then be
   ' opened by calling Phidget_open().  The channel handle must
   ' be released with PhidgetStepper_delete().
   '
   ' stat& (Error Code) will always be 0 if no Errors
   '
   stat& = PhStepCreate&(Dev(indx&).hndl) ' Create new handle
   If stat& <> 0 Then GoTo HandleErr
   
   ' SET SERIAL NUMBER
   stat& = PhSetDeviceSerialNumber(Dev(indx&).hndl, Dev(indx&).hubSn)
   If stat& <> 0 Then GoTo HandleErr

   ' SET ISHUB (0 if hub port is connected to a Phidget module)
   stat& = PhSetIsHubPortDevice(Dev(indx&).hndl, Dev(indx&).isHub)
   If stat& <> 0 Then GoTo HandleErr

   ' SET HUB PORT
   stat& = PhSetHubPort(Dev(indx&).hndl, Dev(indx&).port)
   If stat& <> 0 Then GoTo HandleErr

   ' SET CHANNEL
   stat& = PhSetChannel(Dev(indx&).hndl, Dev(indx&).ch)
   If stat& <> 0 Then GoTo HandleErr
   
   ' Setup the handlers to be run when the device is plugged in,
   ' opened from software, unplugged, closed from software, or
   ' generates an error.
   ' SETUP ATTACH EVENT
   stat& = PhSetOnAttach&(Dev(indx&).hndl, _
               AddressOf PhOnAttach, _
               indx&) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' SETUP DETACH EVENT
   stat& = PhSetOnDetach&(Dev(indx&).hndl, _
               AddressOf PhOnDetach, _
               indx&) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' SETUP ERROR EVENT
   ' Using common Error Handler for all phidget modules.
   stat& = PhSetOnError&(Dev(indx&).hndl, _
               AddressOf PhOnError, _
               indx&) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' ==============================
   ' SETUP ON POSITION CHANGE EVENT
   stat& = PhStepSetOnPositionChangeHandler&( _
               Dev(indx&).hndl, _
               AddressOf PhStepOnPositionChange, _
               indx&) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' SETUP ON STOPPED EVENT
   stat& = PhStepSetOnStoppedHandler&( _
               Dev(indx&).hndl, _
               AddressOf PhStepOnStopped, _
               indx&) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' SETUP ON VELOCITY CHANGE EVENT
   stat& = PhStepSetOnVelocityChangeHandler&( _
               Dev(indx&).hndl, _
               AddressOf PhStepOnVelocityChange, _
               indx&) ' returned with event for this handle
   If stat& <> 0 Then GoTo HandleErr

   ' ===============================
   ' OPEN CHANNEL & WAIT FOR ATTACH with timeout
   stat& = PhOpenWaitForAttachment(Dev(indx&).hndl, timeOutMs&)
   If stat& = 0 Then
      ' Set Initial prameters for stepper
      '
      ' Set Control Mode
      stat& = PhStepSetControlMode&(Dev(indx&).hndl, CONTROL_MODE_STEP)
      If stat& <> 0 Then GoTo HandleErr
      
      ' Set Acceleration
      stat& = PhStepSetAcceleration&(Dev(indx&).hndl, 200000#)
      If stat& <> 0 Then GoTo HandleErr
      
      ' Velocity Limit
      stat& = PhStepSetVelocityLimit&(Dev(indx&).hndl, 15000#)
      If stat& <> 0 Then GoTo HandleErr
      
      ' Current Limit
      stat& = PhStepSetCurrentLimit&(Dev(indx&).hndl, 1#) ' 1 Amp
      If stat& <> 0 Then GoTo HandleErr
      
      ' Set Data Interval
      stat& = PhStepSetDataInterval&(Dev(indx&).hndl, 250&) ' 250ms
      If stat& <> 0 Then GoTo HandleErr
   End If

   GoTo ExitSub
HandleErr:
   Call Form1.DisplayStatus(indx&, stat&, "Stepper")

ExitSub:

End Sub
   
